from tcsoa.gen.StructureManagement._2008_12.services import StructureVerificationService as imp0
from tcsoa.gen.StructureManagement._2010_09.services import StructureVerificationService as imp1
from tcsoa.gen.StructureManagement._2012_02.services import StructureVerificationService as imp2
from tcsoa.gen.StructureManagement._2016_05.services import StructureVerificationService as imp3
from tcsoa.gen.StructureManagement._2012_09.services import StructureService as imp4
from tcsoa.gen.StructureManagement._2007_06.services import PublishByLinkService as imp5
from tcsoa.gen.StructureManagement._2013_05.services import VariantManagementService as imp6
from tcsoa.gen.StructureManagement._2019_06.services import VariantManagementService as imp7
from tcsoa.gen.StructureManagement._2008_03.services import CompositionService as imp8
from tcsoa.gen.StructureManagement._2012_02.services import IncrementalChangeService as imp9
from tcsoa.gen.StructureManagement._2008_05.services import StructureVerificationService as imp10
from tcsoa.gen.StructureManagement._2014_10.services import StructureService as imp11
from tcsoa.gen.StructureManagement._2012_09.services import VariantManagementService as imp12
from tcsoa.gen.StructureManagement._2018_11.services import StructureService as imp13
from tcsoa.gen.StructureManagement._2014_12.services import EffectivityService as imp14
from tcsoa.gen.StructureManagement._2012_10.services import StructureService as imp15
from tcsoa.gen.StructureManagement._2012_09.services import PublishByLinkService as imp16
from tcsoa.gen.StructureManagement._2008_06.services import StructureService as imp17
from tcsoa.gen.StructureManagement._2010_09.services import StructureService as imp18
from tcsoa.gen.StructureManagement._2011_06.services import StructureService as imp19
from tcsoa.gen.StructureManagement._2015_10.services import EffectivityService as imp20
from tcsoa.gen.StructureManagement._2014_10.services import MassUpdateService as imp21
from tcsoa.gen.StructureManagement._2014_06.services import StructureFilterWithExpandService as imp22
from tcsoa.gen.StructureManagement._2016_09.services import PublishByLinkService as imp23
from tcsoa.gen.StructureManagement._2014_06.services import StructureVerificationService as imp24
from tcsoa.gen.StructureManagement._2010_09.services import StructureSearchService as imp25
from tcsoa.gen.StructureManagement._2012_10.services import StructureVerificationService as imp26
from tcsoa.gen.StructureManagement._2013_05.services import StructureVerificationService as imp27
from tcsoa.gen.StructureManagement._2014_12.services import StructureSearchService as imp28
from tcsoa.gen.StructureManagement._2012_09.services import MassUpdateService as imp29
from tcsoa.gen.StructureManagement._2008_05.services import StructureSearchService as imp30
from tcsoa.gen.StructureManagement._2017_05.services import StructureSearchService as imp31
from tcsoa.gen.StructureManagement._2013_05.services import IncrementalChangeService as imp32
from tcsoa.gen.StructureManagement._2010_04.services import StructureSearchService as imp33
from tcsoa.utils import TcService


class StructureVerificationService(TcService):
    accountabilityCheck = imp0.accountabilityCheck
    accountabilityCheck2 = imp1.accountabilityCheck
    accountabilityCheck3 = imp2.accountabilityCheck
    accountabilityCheck4 = imp3.accountabilityCheck2
    accountabilityCheckAsync = imp2.accountabilityCheckAsync
    checkAlignment = imp10.checkAlignment
    compareNetEffectivity = imp1.compareNetEffectivity
    compareNetEffectivity2 = imp2.compareNetEffectivity
    getACFavorite = imp24.getACFavorite
    getAssignmentComparisonDetails = imp2.getAssignmentComparisonDetails
    getAssignmentComparisonDetails2 = imp26.getAssignmentComparisonDetails
    getAttrGrpsAndFormsComparisonDetail = imp3.getAttrGrpsAndFormsComparisonDetail
    getAttributeGroupsAndFormsComparisonDetails = imp27.getAttributeGroupsAndFormsComparisonDetails
    getComparisonSummaries = imp26.getComparisonSummaries
    getConnectedObjectsComparisonDetails = imp27.getConnectedObjectsComparisonDetails
    getDescendentComparisonDetails = imp2.getDescendentComparisonDetails
    getPartitionComparisonDetails = imp2.getPartitionComparisonDetails
    getPredecessorComparisonDetails = imp2.getPredecessorComparisonDetails
    getPropertyComparisonDetails = imp2.getPropertyComparisonDetails
    manageACFavorites = imp24.manageACFavorites
    propagateProperties = imp2.propagateProperties


class StructureService(TcService):
    add = imp4.add
    cloneStructure = imp11.cloneStructure
    cloneStructureAsync = imp11.cloneStructureAsync
    cloneStructureExpandOrUpdate = imp11.cloneStructureExpandOrUpdate
    createInterchangeableGroups = imp13.createInterchangeableGroups
    cutItems = imp15.cutItems
    duplicate = imp17.duplicate
    duplicate2 = imp18.duplicate2
    duplicate3 = imp19.duplicate3
    expandOrUpdateDuplicateItems = imp17.expandOrUpdateDuplicateItems
    expandOrUpdateDuplicateItems2 = imp18.expandOrUpdateDuplicateItems2
    expandOrUpdateDuplicateItems3 = imp19.expandOrUpdateDuplicateItems3
    packOrUnpack = imp18.packOrUnpack
    setClosureRuleVariablesAndValues = imp19.setClosureRuleVariablesAndValues
    toggleOccurrenceSuppression = imp11.toggleOccurrenceSuppression
    togglePrecision = imp11.togglePrecision
    unloadBelow = imp19.unloadBelow
    validateOccurrenceConditions = imp4.validateOccurrenceConditions
    validateParentChildConditions = imp4.validateParentChildConditions
    validateStructureItemIds = imp17.validateStructureItemIds
    validateStructureItemIds2 = imp18.validateStructureItemIds2
    validateStructureItemIds3 = imp19.validateStructureItemIds3


class PublishByLinkService(TcService):
    addTargets = imp5.addTargets
    completenessCheck = imp5.completenessCheck
    createLinks = imp5.createLinks
    deleteLinkForSource = imp5.deleteLinkForSource
    deleteLinksForSource2 = imp16.deleteLinksForSource2
    deleteTargetsFromLink = imp5.deleteTargetsFromLink
    findLinesWithSameLogicalIdentity = imp5.findLinesWithSameLogicalIdentity
    findSourceInWindow = imp5.findSourceInWindow
    findSourcesInWindow = imp23.findSourcesInWindow
    findTargetsInWindow = imp5.findTargetsInWindow
    getSourceTopLevel = imp5.getSourceTopLevel
    publishData = imp5.publishData


class VariantManagementService(TcService):
    applyBOMVariantRules = imp6.applyBOMVariantRules
    applyBOMVariantRules2 = imp7.applyBOMVariantRules2
    createAndSubstituteVariantItem = imp12.createAndSubstituteVariantItem
    createVariantItem = imp12.createVariantItem
    getBOMVariantRules = imp6.getBOMVariantRules
    getBOMVariantRules2 = imp7.getBOMVariantRules2
    getVariantExpressionsMatchInfo = imp6.getVariantExpressionsMatchInfo
    setBOMVariantRules = imp6.setBOMVariantRules
    setBOMVariantRules2 = imp7.setBOMVariantRules2


class CompositionService(TcService):
    assignChildLines = imp8.assignChildLines


class IncrementalChangeService(TcService):
    carryOver = imp9.carryOver
    removeIncrementalChanges = imp32.removeIncrementalChanges


class EffectivityService(TcService):
    createOccurrenceEffectivities = imp14.createOccurrenceEffectivities
    createReleaseStatusEffectivity = imp14.createReleaseStatusEffectivity
    editOccurrenceEffectivity = imp20.editOccurrenceEffectivity
    editReleaseStatusEffectivity = imp20.editReleaseStatusEffectivity
    removeOccurrenceEffectivities = imp20.removeOccurrenceEffectivities
    removeReleaseStatusEffectivity = imp20.removeReleaseStatusEffectivity


class MassUpdateService(TcService):
    executeMarkupChanges = imp21.executeMarkupChanges
    expandOneLevel = imp21.expandOneLevel
    getImpactedObjectDetails = imp21.getImpactedObjectDetails
    getImpactedObjects = imp21.getImpactedObjects
    getMarkupChangesForUpdate = imp21.getMarkupChangesForUpdate
    getRevisionRules = imp21.getRevisionRules
    manageImpactedObjectUpdates = imp21.manageImpactedObjectUpdates
    massGetAffectedParents = imp29.massGetAffectedParents
    massUpdateExecutionECN = imp29.massUpdateExecutionECN
    massUpdateExecutionECR = imp29.massUpdateExecutionECR
    updateImpactedObjects = imp21.updateImpactedObjects
    updateImpactedObjectsEnd = imp21.updateImpactedObjectsEnd
    updateImpactedObjectsStart = imp21.updateImpactedObjectsStart
    validateChangeObjectForMassUpdate = imp21.validateChangeObjectForMassUpdate


class StructureFilterWithExpandService(TcService):
    expandAndSearch = imp22.expandAndSearch


class StructureSearchService(TcService):
    getAssemblyBoundingBox = imp25.getAssemblyBoundingBox
    getStructureChanges = imp28.getStructureChanges
    isSpatialDataAvailable = imp25.isSpatialDataAvailable
    nextSearch = imp30.nextSearch
    nextSearch2 = imp31.nextSearch2
    startSearch = imp30.startSearch
    startSearch2 = imp33.startSearch
    startSearch3 = imp25.startSearch
    startSearch4 = imp31.startSearch2
    stopSearch = imp30.stopSearch
    stopSearch2 = imp31.stopSearch2



from __future__ import annotations

from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from typing import Dict, List


@dataclass
class GetSearchSettingsInput(TcBaseObj):
    """
    Input structure for the getSearchSettings service which is used to set the inputs for which values are needed from
    server.
    
    This input structure uses a map to provide a way for the client to request specific display values for multiple
    preference in bulk. The Key for this map is the dynamic set of preference names to request display values for. The
    Value is the specific list of internal values to translate to display values.
    
    :var inputSettings: Input map(string, list<string>) for getting the search settings. Supported keys and values are
    (this set will be expanded in future releases)
    --------------------------------------------------------------------------------------------------
    Key                                                                   |                Values
    --------------------------------------------------------------------------------------------------
    AWC_Limited_Filter_Categories_Expanded |     List of internal property names  
                                                                          |     <type_name>.<property name>] Ex: 
                                                                          |     WorkspaceObject.object_type
    --------------------------------------------------------------------------------------------------
    Default_Quick_Access_Query                       |     Internal name of the quick access query
    --------------------------------------------------------------------------------------------------
    """
    inputSettings: StringVectorMap = None


@dataclass
class GetSearchSettingsResponse(TcBaseObj):
    """
    The input key and its corrosponding output values.
    
    :var outputValues: Output values map(string, list of<string>) that contain the input key and its corresponding
    outputs.
    ---------------------------------------------------------------------------------------------------
    Key                                                                     |       Values
    ---------------------------------------------------------------------------------------------------
    AWC_Limited_Filter_Categories_Expanded   |  A List of [display names of the properties] Ex: For 
                                                                            |  WorkspaceObject.object_type as input the
    output 
                                                                            |  would be "Type"
    ---------------------------------------------------------------------------------------------------
    Default_Quick_Access_Query                         |  Display name of the quick access query
    ---------------------------------------------------------------------------------------------------
    """
    outputValues: StringVectorMap = None


"""
A map of string to list of strings.
"""
StringVectorMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class PublishColumnConfigInfo(TcBaseObj):
    """
    'PublishColumnConfigInfo' structure represents the parameters required to publish the column configuration.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var columnConfiguration: The Fnd0ColumnConfiguration object, that stores all preferences that needs to be created
    as site preferences.
    """
    clientId: str = ''
    columnConfiguration: BusinessObject = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ApprSearchCriteriaInClass
from tcsoa.gen.Rdv._2012_09.ContextManagement import CreateSearchCriteriaScpResponse, CreateSearchSCOResponse, UpdateSearchSCOInputInfo, GetICSClassNamesResponse, CreateSearchSCOInputInfo, UpdateSearchSCOResponse, CreateSearchCriteriaScopeInfo, CreateFormAttrSearchCriteriaResponse, CreateFormAttrSearchCriteriaInputInfo
from typing import List
from tcsoa.utils import TcService


class ContextManagementService(TcService):

    @classmethod
    def updateSearchSCO(cls, inputs: List[UpdateSearchSCOInputInfo]) -> UpdateSearchSCOResponse:
        """
        Updates the Search Structure Context Object (SearchSCO) based on the inputs attributes. It sets the following
        properties on SearchSCO object which is to be modified: Product Item Revision, Revision Rule, Variant Rule,
        Work parts selected, Search Criteria Group, Target and Background BOMLine objects, and result status from the
        input structure. This SearchSCO will contain the Item, Item revisions, Target BOMLine objects, Background
        BOMLine objects. The operation is designed to support multiple SearchSCOs creation in a single call. This
        operation first checks for the local ownership of the object to be updated. This operation will fail if null or
        incorrect reference to existing SearchSCO object is passed in the input.
        
        Use cases:
        You can update an SCO object of type SearchStructureContext using 'updateSearchSCO' operation by providing the
        'UpdateSearchSCOInputInfo' structure.
        - Create an SCO, object of SearchStructureContext, using the 'createSearchSCO' operation.
        - Retrieve the reference to SearchStructureContext returned from above step.
        - Modify the required search criteria and populate the 'UpdateSearchSCOInputInfo' structure.
        - Call 'updateSearchSCO' which will modify the existing SearchStructureContext object.
        
        """
        return cls.execute_soa_method(
            method_name='updateSearchSCO',
            library='Rdv',
            service_date='2012_09',
            service_name='ContextManagement',
            params={'inputs': inputs},
            response_cls=UpdateSearchSCOResponse,
        )

    @classmethod
    def createFormAttrSearchCriteria(cls, inputs: List[CreateFormAttrSearchCriteriaInputInfo]) -> CreateFormAttrSearchCriteriaResponse:
        """
        Creates a list of Form Search Criteria objects based on the input parameters. It uses the following inputs from
        the input structure
        - Form type
        - Parent Type
        - Name of the property
        - Type of the property
        - Relation type
        - Operator and 
        - Search value
        
        
        Form Search Criteria object is created as part of creation of the VisStructureContext object to persist form
        attribute search criteria. 
        It is mandatory to provide all the input parameters. This operation will fail if null is provided for any of
        the string input parameters. The Form type or Relation type input should be valid string representing the name
        of a type in Teamcenter database. Empty string for Form type or Relation type input will cause the operation to
        fail.
        
        Use cases:
        1. Creating an Structure Context Oject(SCO)
        While pesristing the search criterias in Structure Context Oject, 'createFormAttrSearchCriteria' operation is
        called to persist the Form serach criteria related information . You can create this object by providing the
        'CreateFormAttrSearchCriteriaInputInfo' structure. This form search criteria object is wrapped in Appearance
        Search Criteria Group object and stored in StructureContext object.
        
        Exceptions:
        >Following are some possible errors returned in 'ServiceData':
        - 202017        The class cannot be instantiated.
        - 39007        The specified name is invalid for a type.
        - 39014        The specified type does not exist.
        
        """
        return cls.execute_soa_method(
            method_name='createFormAttrSearchCriteria',
            library='Rdv',
            service_date='2012_09',
            service_name='ContextManagement',
            params={'inputs': inputs},
            response_cls=CreateFormAttrSearchCriteriaResponse,
        )

    @classmethod
    def createSearchCriteriaScope(cls, inputs: List[CreateSearchCriteriaScopeInfo]) -> CreateSearchCriteriaScpResponse:
        """
        This operation creates the Fnd0ApprSchCriteriaScpAttr object (ScpSrchCriteria) based on the inputs supplied. It
        creates a Scope Search Criteria object and stores the scope of the search. Scope Search Criteria object is
        created as part of the creation of the SearchStructureContext object to persist the scope of the search.
        If the input contains APNs or BOMLine objects, then these are saved in either appearance path nodes list or
        occurrence list property depending on the value of RDV_CREATE_SCO_WITHOUT_APN preference. If the input contains
        appearance groups, these are stored in occurrence group list property of ScpSrchCriteria object.
        
        Use cases:
        1. Creating a Search Search Criteria
        While persisting the search criteria in Search Structure Context object, 'createSearchCriteriaScope' operation
        is called to persist the scope of the search. User can create this object by providing the
        'CreateSearchCriteriaScopeInfo' structure. This scope search criteria object is wrapped in Appearance Search
        Criteria Group object and stored in SearchStructureContext object. 
        """
        return cls.execute_soa_method(
            method_name='createSearchCriteriaScope',
            library='Rdv',
            service_date='2012_09',
            service_name='ContextManagement',
            params={'inputs': inputs},
            response_cls=CreateSearchCriteriaScpResponse,
        )

    @classmethod
    def createSearchSCO(cls, inputs: List[CreateSearchSCOInputInfo]) -> CreateSearchSCOResponse:
        """
        Creates the SearchStructureContext object (SearchSCO) based on the inputs supplied. It creates a SearchSCO
        object and then sets the following properties on SearchSCO object created: Product Item Revision, Revision
        Rule, Variant Rule, Work parts selected, Search Criteria Group, Target and Background BOMLine objects from the
        input structure, result stored status, object shared status. This SearchSCO will contain the Item, Item
        revisions, Target BOMLine objects, Background BOMLine objects. The operation is designed to support multiple
        SearchSCOs creation in a single call. The operation will initially create the SearchSCO object using the name,
        type and description. Subsequently it would set the additional parameters supplied through the input structure.
        SearchSCO object would still be created and saved even if setting of the additional parameters is not
        successful.
        
        Use cases:
        1. Create a Search SCO
        You can create a new SCO object of type SearchStructureContext using 'createSearchSCO' operation, by providing
        the 'CreateSearchSCOInputInfo' structure.
        """
        return cls.execute_soa_method(
            method_name='createSearchSCO',
            library='Rdv',
            service_date='2012_09',
            service_name='ContextManagement',
            params={'inputs': inputs},
            response_cls=CreateSearchSCOResponse,
        )

    @classmethod
    def getICSClassNames(cls, searchCriteriaInClass: List[ApprSearchCriteriaInClass]) -> GetICSClassNamesResponse:
        """
        Creates a list of ICS class names for the ApprSearchCriteriaInClass objects passed in the input. Object of ICS
        class is stored inside ApprSearchCriteriaInClass object and this is persisted along with other search criteria
        in Structure Context object (SCO). During replay of SCO, in order to reconstruct the Classification object this
        operation is called to get the class name. Using this class name ICS object is recreated at client side. This
        method is required because the classification object cannot be retrieved in its original format from the SCO
        object.
        
        Use cases:
        1. Replaying an SCO
        You can reconstruct the classification object stored in SCO using the class name returned from the
        'getICSClassNames' operation by providing the reference to the ApprSearchCriteriaInClass object.
        - Create an SCO, object of StructureContext, using the 'createSCO' operation.
        - Retrieve the reference to StructureContext returned from above step.
        - Fetch the reference of ApprSearchCriteriaInClass stored in the Appearance Grouping object.
        - Provide the object retrieved above to 'getICSClassNames' operation.
        
        
        
        Exceptions:
        >Following are some possible errors returned in 'ServiceData':
        - 202023        The attribute of object cannot be retrieved
        - 202024        The object cannot be found.
        
        """
        return cls.execute_soa_method(
            method_name='getICSClassNames',
            library='Rdv',
            service_date='2012_09',
            service_name='ContextManagement',
            params={'searchCriteriaInClass': searchCriteriaInClass},
            response_cls=GetICSClassNamesResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, POM_imc, TransferOptionSet
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ImportObjectsFromBriefcaseInput(TcBaseObj):
    """
    The struct for defining the input arguments required by the importObjectsFromBriefcase operation.
    
    :var bczFileTicket: The FMS file ticket for the source briefcase file. User needs to upload the briefcase file to
    the FMS transient volume to generate the file ticket.
    :var transferOptionSet: The Transfer Option Set object used for data scoping.
    :var overrideOptions: The name and value pairs of options that will override the same options defined in the input
    Transfer Option Set.
    :var sessionOptions: The name and value pair options to control the export session behavior. The supported options
    are:
    1. dry_run: valid values are True/False, specifies that the briefcase import report is generated which has
    discrepancy if any.
    2. ValidateImport: valid values are True/False, specifies that the TCXML is validated against any discrepancy.
    3. validateXMLBeforeXslt: valid values are True/False, specifies that the TCXML is validated before mapping is done.
    """
    bczFileTicket: str = ''
    transferOptionSet: TransferOptionSet = None
    overrideOptions: List[NameAndValue] = ()
    sessionOptions: List[NameAndValue] = ()


@dataclass
class NameAndValue(TcBaseObj):
    """
    NameAndValue structure represents an option name-value pair.
    
    :var optionName: The name of the option name-value pair.
    :var optionValue: The value of the option name-value pair.
    """
    optionName: str = ''
    optionValue: str = ''


@dataclass
class ExportObjectsToBriefcaseInput(TcBaseObj):
    """
    The struct for defining the input arguments required by the exportObjectsToBriefcase operation.
    
    :var rootObjects: The root objects to be exported.
    :var targetSite: The target site.
    :var transferOptionSet: The Transfer Option Set object used for data scoping.
    :var overrideOptions: The name and value pairs of options that will override the same options defined in the input
    Transfer Option Set.
    :var sessionOptions: The name and value pair options to control the export session behavior. The supported options
    are:
    1. pkgDatasetName: valid values are "Dataset name", specifies the output briefcase Dataset name.
    2. revRule: valid values are "Revision Rule name", specifies the Reivsion Rule that will be applied on the root
    object for configured BOM export.
    3. varRule: valid values are "Variant Rule name", specifies the Variant Rule that associated to the root object for
    configured BOM export.
    4. dry_run: valid values are True/False, specifies that the briefcase export report is generated which has
    discrepancy if any.
    5. validateXML: valid values are True/False, specifies that the TCXML is validated against any discrepancy.
    6. validateXMLBeforeXslt: valid values are True/False, specifies that the TCXML is validated before mapping is done.
    7. modified_objects_only: valid values are True/False, specifies the option for delta export.
    8. opt_sync_previous_delta: valid values are True/False, specifies the option for force re-export.
    9. OwnershipTransfer: valid values are True/False, specifies the option to do ownership transfer.
    10. objsForOwnXfer: valid values are in the format of "<site-id>:<object-uid1,object-uid2&hellip;>", specifies the
    objects for ownership transfer.
    11. processUnconfiguredVariants: valid values are True/False, specifies the option to process the BOMLines that are
    configured out by the Variant Rule.
    12. processUnconfiguredByOccEff: valid values are True/False, specifies the option to process the BOMLines that are
    configured out by Occurrence Effectivity.
    13. processSuppressedOcc: valid values are True/False, specifies the option to process the suppressed BOMLines.
    14. processUnconfiguredChanges: valid values are True/False, specifies the option to process the BOMLines that are
    configured out by Incremental Change.
    15. ContinueOnError: valid values are True/False, specifies the option to continue the export on partial error.
    """
    rootObjects: List[BusinessObject] = ()
    targetSite: POM_imc = None
    transferOptionSet: TransferOptionSet = None
    overrideOptions: List[NameAndValue] = ()
    sessionOptions: List[NameAndValue] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.StructureManagement._2012_02.StructureVerification import EquivalentLines2, PartialMatchCriteria2, ToolRequirementComparisonResult, GetActivitiesComparisonDetailsResponse
from typing import List
from tcsoa.utils import TcService


class StructureVerificationService(TcService):

    @classmethod
    def getToolRequirementComparisonDetails(cls, sourceObjects: List[BusinessObject], targetObjects: List[BusinessObject]) -> ToolRequirementComparisonResult:
        """
        Compares the tool requirements and fetches their comparison data. Refer to the complete documentation of
        ToolRequirementComparisonData on the data that are compared and how they are represented.
        """
        return cls.execute_soa_method(
            method_name='getToolRequirementComparisonDetails',
            library='Internal-StructureManagement',
            service_date='2012_02',
            service_name='StructureVerification',
            params={'sourceObjects': sourceObjects, 'targetObjects': targetObjects},
            response_cls=ToolRequirementComparisonResult,
        )

    @classmethod
    def getActivitiesComparisonDetails(cls, equivalentObjects: List[EquivalentLines2], comparisonCriteria: PartialMatchCriteria2) -> GetActivitiesComparisonDetailsResponse:
        """
        Returns the details of any differences between activities for the supplied source and target objects.
        """
        return cls.execute_soa_method(
            method_name='getActivitiesComparisonDetails',
            library='Internal-StructureManagement',
            service_date='2012_02',
            service_name='StructureVerification',
            params={'equivalentObjects': equivalentObjects, 'comparisonCriteria': comparisonCriteria},
            response_cls=GetActivitiesComparisonDetailsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Awb0ProductContextInfo
from tcsoa.gen.Internal.ActiveWorkspaceVis._2014_11.OccurrenceManagement import GetVisBookmarkInfoResponse, SaveVisBookmarkInfoResponse, SaveVisBookmarkInput
from typing import List
from tcsoa.utils import TcService


class OccurrenceManagementService(TcService):

    @classmethod
    def getVisBookmarkInfo(cls, productContextInfos: List[Awb0ProductContextInfo]) -> GetVisBookmarkInfoResponse:
        """
        In order to take the user to the context where he left off, viewer state information (e.g. camera, occurrence
        visibility, etc.) are stored to the current Awb0AutoBookmark for the given Awb0ProductContextInfo.  This
        operation gets the visualization information associated to the  Awb0Autobookmark for the given
        Awb0ProductContextInfo.
        
        Use cases:
        The user searches for a product clicks on the viewer tab from the Active Content  Experience(ACE) location in
        Active Workspace.
        - A launch file(.vvi) containing the Awb0ProductContextInfo information gets created. This file is uploaded to
        the visualization server.
        - Viewer which is running in the visualization server reads this launch file and loads the occurrences for the
        given Awb0ProductContextInfo.
        - Viewer invokes 'getVisBookmarkInfo', downloads the file containing the visualization information using the
        read tickets returned in 'VisBookmarkInfoResponse', reads the file and sets the visualization state that would
        take the user back to where he left off.
        
        """
        return cls.execute_soa_method(
            method_name='getVisBookmarkInfo',
            library='Internal-ActiveWorkspaceVis',
            service_date='2014_11',
            service_name='OccurrenceManagement',
            params={'productContextInfos': productContextInfos},
            response_cls=GetVisBookmarkInfoResponse,
        )

    @classmethod
    def saveVisBookmarkInfo(cls, saveBookmarkInfos: List[SaveVisBookmarkInput]) -> SaveVisBookmarkInfoResponse:
        """
        When a user changes various visibility state (e.g. camera, occurrence visibility, etc) that state need to be
        saved/associated to a bookmark. This operation is used to associate the current visualization state information
        to the current Awb0AutoBookmark for the Awb0ProductContextInfo.
        
        Use cases:
        The user searches for a product clicks on the viewer tab from the Active Content  Experience(ACE) location in
        Active Workspace.
        - A launch file(.vvi) containing the Awb0ProductContextInfo gets created. This file is uploaded to the
        visualization server.
        - Viewer which is running in the visualization server reads this launch file and loads the occurrences for the
        given Awb0ProductContextInfo.
        - User changes occurrence visibility or camera angle, the viewer writes the changed information into a file. 
        - The file is now uploaded by viewer to the transient volume, invokes 'saveVisBookmarkInfo' to save the changed
        viewer information as ImanFile object and associate it to the current Awb0AutoBookmark for the
        Awb0ProductContextInfo.
        
        """
        return cls.execute_soa_method(
            method_name='saveVisBookmarkInfo',
            library='Internal-ActiveWorkspaceVis',
            service_date='2014_11',
            service_name='OccurrenceManagement',
            params={'saveBookmarkInfos': saveBookmarkInfos},
            response_cls=SaveVisBookmarkInfoResponse,
        )



from __future__ import annotations

from typing import Dict, List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ItemIdRecordsProperties(TcBaseObj):
    """
    'ItemIdRecordsProperties' structure which contains itemids and MFK values to be registered in Item registry.
    
    :var itemId: The ID of the Item to be registered.
    :var context: Not being used.
    :var keyValue: MFK key value for a input item.
    :var domain: Domain name i.e. class name for a input item.
    :var registrationSite: Siteid of the owning site of given itemId.
    """
    itemId: str = ''
    context: str = ''
    keyValue: str = ''
    domain: str = ''
    registrationSite: int = 0


@dataclass
class PublicationRecordProperties(TcBaseObj):
    """
    'PublicationRecordProperties' structure which contains attributes of Published object which is published to ODS.
    
    :var owningSiteId: siteid of owning site of published object.
    :var objectTag: Tag of the published object.
    :var creationDate: Date of creation of published Object.
    :var pubDate: Date of publication of published Object.
    :var releaseStatus: Relase status if any of published Object is released.
    :var domainName: MFK domain name of published Object.
    :var keyValue: MFK value of published Object.
    :var customAttributes: It is a string Map of custom attributes of published Object and ODS custom attribute.  This
    is used in case of extensible ODS feature.
    :var l10NPropertiesInfo: List of localized attributes of published Object.
    :var l10NLocalizedValues: List of corressponding localized attribute values of published Object.
    :var localeCountStr: Formated string which contains value count of locales for each localized attribute.
    :var objectId: ID of the published object.
    :var revisionId: Revision id of the published object.
    :var objectName: Object name of published Object.
    :var objectDesc: Object description of published Object.
    :var objectClass: Object class name of published Object.
    :var objectType: Object type name of published Object.
    :var owningUserId: Owning user id of published Object.
    :var owningGroupName: Owning group name of published Object.
    """
    owningSiteId: int = 0
    objectTag: str = ''
    creationDate: datetime = None
    pubDate: datetime = None
    releaseStatus: List[str] = ()
    domainName: str = ''
    keyValue: str = ''
    customAttributes: MapDataMfk = None
    l10NPropertiesInfo: List[str] = ()
    l10NLocalizedValues: List[str] = ()
    localeCountStr: str = ''
    objectId: str = ''
    revisionId: str = ''
    objectName: str = ''
    objectDesc: str = ''
    objectClass: str = ''
    objectType: str = ''
    owningUserId: str = ''
    owningGroupName: str = ''


@dataclass
class BoMfkInfo(TcBaseObj):
    """
    'BoMfkInfo' structure contains mfk value for which item id record to be queried from the Item registry.
    
    
    :var itemId: The ID of the Item to be unregistered.
    :var keyValue: MFK key value for a input item to be unregistered.
    :var domain: Domain name of the MultifieldKey i.e. class name for a input item.
    """
    itemId: str = ''
    keyValue: str = ''
    domain: str = ''


@dataclass
class ClientInfoProperties(TcBaseObj):
    """
    'ClientInfoProperties' structure contains requesting ODS client information.
    
    :var clientSiteId: Requesting client siteid.
    :var targetServerSiteId: Target server siteid.
    :var userName: Logged in username of requesting client.
    :var osUserName: OS logged in username of requesting client node.
    
    :var groupName: Group name of userid of requesting client.
    
    :var roleName: Role name of userid of requesting client.
    
    :var customAttributeNames: String map of custom attributes of published Object and ODS custom attribute. This is
    used in case of extensible ODS feature.
    
    :var clientNode: Node name of requesting client.
    :var clientReleaseVersion: Teamcenter release version of requesting client.
    :var clientMinorReleaseVersion: Minor release version of requesting client.
    :var clientServiceReleaseVersion: Teamcenter release version of requesting client server.
    
    :var clientMaintPackVersion: Teamcenter maintaince pack version of requesting client.
    
    :var clientOdsMajorVersion: Client ODS major version.
    :var clientOdsMinorVersion: client ODS minor version.
    :var userId: Logged in userid of requesting client.
    """
    clientSiteId: int = 0
    targetServerSiteId: int = 0
    userName: str = ''
    osUserName: str = ''
    groupName: str = ''
    roleName: str = ''
    customAttributeNames: MapDataMfk = None
    clientNode: str = ''
    clientReleaseVersion: int = 0
    clientMinorReleaseVersion: int = 0
    clientServiceReleaseVersion: int = 0
    clientMaintPackVersion: int = 0
    clientOdsMajorVersion: int = 0
    clientOdsMinorVersion: int = 0
    userId: str = ''


"""
Map Data
"""
MapDataMfk = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ValidationData, Dataset, ValidationResult, WorkspaceObject, ItemRevision
from enum import Enum
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class OneResultEvaluated(TcBaseObj):
    """
    OneResultEvaluated lists evaluation status for one validation result. Validation result object may be NULL if said
    validation is not performed.
    
    :var status: Validation result status defined in enum validationRunStatus.
    :var partDataset: Pointer to the validation target object.
    :var checker: Pointer to a validation checker object.
    :var result: Pointer to a ValidationResult object that was created for the specified validation checker and
    validation target.
    :var checkPassed: When TRUE, it indicates that the validation checker for the validation target is passed according
    to validation rule. When FALSE, the check failed according to the validation rules.
    """
    status: validationRunStatus = None
    partDataset: Dataset = None
    checker: ValidationData = None
    result: ValidationResult = None
    checkPassed: bool = False


@dataclass
class OneSetResultsEvaluated(TcBaseObj):
    """
    OneSetResultsEvaluated lists evaluation statuses for a set of validation results. Compound result status is TRUE if
    all results in current set passed validation rule verification. Otherwise, the status is FALSE.
    
    :var evaluatedResults: Evaluation statuses for each of validation result in a set.
    :var compoundResult: When TRUE, all validation results in the set satisfy validation rule. When FALSE, at least one
    validation result failed according to validation rule.
    :var clientId: Unique client identifier string.
    """
    evaluatedResults: List[OneResultEvaluated] = ()
    compoundResult: bool = False
    clientId: str = ''


@dataclass
class ValidationExpressionInfo(TcBaseObj):
    """
    The ValidationExpressionInfo structure represents input requirement formula for operation
    parseRequirementExpressions.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify returned ValidationParserOutput
    elements and partial errors associated with this input ValidationExpressionInfo.
    :var requirementExpression: A requirement formula string to be checked for syntax and to be parsed.
    """
    clientId: str = ''
    requirementExpression: str = ''


@dataclass
class ValidationParserOutput(TcBaseObj):
    """
    The ValidationParserOutput structure contains a list of variables names and a list of functions names parsed from
    the requirement formula string by operation parseRequirementExpressions.
    
    :var clientId: Identifying string from the source ValidationExpressionInfo.
    :var variableNames: A list of variables names parsed from requirement formula string.
    :var functionNames: A list of function names parsed from requirement formula string.
    """
    clientId: str = ''
    variableNames: List[str] = ()
    functionNames: List[str] = ()


@dataclass
class ValidationParserResponse(TcBaseObj):
    """
    The ValidationParserResponse structure holds the response from parseRequirementExpressions.
    
    :var output: List of names of variables and functions in the input requirement formula string in
    ValidationExpressionInfo.
    :var serviceData: The service data.
    """
    output: List[ValidationParserOutput] = ()
    serviceData: ServiceData = None


@dataclass
class EvaluateValidationResultsWithRulesInfo(TcBaseObj):
    """
    The EvaluateValidationResultsWithRulesInfo structure is the main input for evaluateValidationResultsWithRules
    operation to evaluate validation results. It may contain such workspace objects as items or item revisions or
    datasets or folders as well as validation rule set item revision or checkers. Client has the option to provide
    either validation rule item revision or list of checkers. When neither validation rule item revision nor checkers
    is provided, evaluation is performed without rule (in which case all existing validation results are evaluated as
    mandatory checks and all must PASS).
    
    :var selectedWSOs: List of Folder, Item, ItemRevision, Dataset to be evaluated.
    :var selectedCheckers: List of checkers used for evaluation.  The checker must be a NXValDataRevision or a subtype
    of it. This can be specified together with 'datasetTypes' when no rule set item revision is provided.  If a rule
    set item revision is already specified by 'ruleItemRevision', the checkers specified here will be ignored.
    :var datasetTypes: List of dataset types. This must be specified if a list of checkers is used for evaluation.  If
    it is not specified but the checkers are provided, then the checkers are ignored by this operation. Its array
    length does not need to match the list of selected checkers. All dataset types in this array applies to all
    checkers in 'selectedCheckers'. For example,if user provided checker1, checker2 and checker3 in 'selectedCheckers',
    then provided dataset type A and data set type B in 'datasetTypes', it means that the dataset type A and datatype B
    must have passed validation results for checker 1, 2 and 3.
    :var ruleItemRevision: Validation rule set item revision containing a ValidationRuleSet dataset. Can be NULL if a
    list of checkers is used for evaluation.
    :var clientId: A unique string supplied by the caller. This ID is used to identify returned OneSetResultsEvaluated
    elements and Partial Errors associated with this input EvaluateValidationResultsWithRulesInfo.
    """
    selectedWSOs: List[WorkspaceObject] = ()
    selectedCheckers: List[ValidationData] = ()
    datasetTypes: List[str] = ()
    ruleItemRevision: ItemRevision = None
    clientId: str = ''


@dataclass
class EvaluateValidationResultsWithRulesResponse(TcBaseObj):
    """
    The EvaluateValidationResultsWithRulesResponse structure holds the response from evaluateValidationResultsWithRules
    operation.
    
    :var evaluatedResultsList: List of sets of validation results that evaluated for each input
    EvaluateValidationResultsWithRulesInfo.
    :var serviceData: The service data.
    """
    evaluatedResultsList: List[OneSetResultsEvaluated] = ()
    serviceData: ServiceData = None


class validationRunStatus(Enum):
    """
    validationRunStatus lists validation result evaluation statuses.
    """
    upToDate = 'upToDate'
    outOfDatePartChanged = 'outOfDatePartChanged'
    outOfDateRequirementChanged = 'outOfDateRequirementChanged'
    verifyParameterFailed = 'verifyParameterFailed'
    missingReportLog = 'missingReportLog'
    notRun = 'notRun'
    notRequired = 'notRequired'



from __future__ import annotations

from tcsoa.gen.ImportExport._2012_09.FileImportExport import ImportFromApplicationInputData3
from tcsoa.utils import TcService


class FileImportExportService(TcService):

    @classmethod
    def importSpecAsync(cls, input: ImportFromApplicationInputData3) -> None:
        """
        This operation imports the selected Requirement Specification word document asynchronously to Teamcenter.
        Following are the supported input application formats:
        - MSWordXML- This mode provides the capability of importing MSWord document to Teamcenter.
        
        
        
        Use cases:
        User selects or navigates inside a folder and clicks on the Import Specification button in the toolbar. This
        will show a panel with options to browse a Word file, a dropdown lists to select a Specification type and a
        SpecElement type. If user clicks on Import button, the Word document is imported in Teamcenter. User receives a
        notification in active workspace client after the import completes.
        """
        return cls.execute_soa_method(
            method_name='importSpecAsync',
            library='Internal-ImportExport',
            service_date='2017_05',
            service_name='FileImportExport',
            params={'input': input},
            response_cls=None,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Item, BusinessObject, Form, Folder, ItemRevision
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class MasterFormPropertiesInfo(TcBaseObj):
    """
    'MasterFormPropertiesInfo' specifies the property values to be set on a form.
    
    :var form: Form object to use as the new master form for the related object and will be populated with the property
    values. Not required. If not specified, a master form will be created and populated with the property values.
    :var propertyValueInfo: The list of 'PropertyNameValueInfo's.
    """
    form: Form = None
    propertyValueInfo: List[PropertyNameValueInfo] = ()


@dataclass
class PropertyNameValueInfo(TcBaseObj):
    """
    This structure specifies a property name and the related value(s) to apply to the property.
    
    :var propertyName: The property name.
    :var propertyValues: The list of values for the property.
    """
    propertyName: str = ''
    propertyValues: List[str] = ()


@dataclass
class RelatedObjectInfo(TcBaseObj):
    """
    Structure for specifying 'RelatedObjectInfo'.
    
    :var relatedObject: A reference representing the other side object used to propagate the relation while applying
    the deep copy rules. This object could be a newly created object or an existing object in the system.
    :var action: An integer representing the action that specifies the how to copy the 'relatedObject' as the other
    side object while applying the deep copy rules.  Valid values are : 
    CopyAsObjectType = 0, (A new copy of the object represented by otherSideObjectTag will be created and while
    propagating the relations) 
    CopyAsRefType = 1, (The same object represented by 'otherSideObject' will be used to propagate the relations)
     DontCopyType =2, (The relation will not be propagated.) 
    RelateToLatest = 3, (If the other side object is an ItemRevision, then the latest version of the revision will be
    used while propagating the relation) 
    ReviseAndRelateToLatest = 4 (If the other side object is an ItemRevision, it will be revised and the newly created
    revision will be used while propagating the relations)
    :var isSecondary: A flag indicating whether the 'relatedObject' is participating as a secondary object in the
    relation.
    :var nRdeepCopyOutput: Related object info for named reference list.
    """
    relatedObject: BusinessObject = None
    action: int = 0
    isSecondary: bool = False
    nRdeepCopyOutput: List[RelatedObjectInfoForNR] = ()


@dataclass
class RelatedObjectInfoForNR(TcBaseObj):
    """
    Structure specifying the results of the deep copy of attached named reference objects.
    
    :var relatedObject: Reference to the related object, can either be a new object or an existing object depending on
    the action performed.
    :var action: An integer representing the action that specifies how to the deep copy rules were applied. Valid
    values are :
    CopyAsObjectType = 0, (A new copy of the object represented by otherSideObjectTag was created and while
    propagatring the relations) 
    CopyAsRefType = 1, (The same object represented by otherSideObjectTag was used to propagate the relations)
    
    :var isSecondary: A flag indicating whether the 'relatedObject' is participating as a secondary object in the
    relation.
    """
    relatedObject: BusinessObject = None
    action: int = 0
    isSecondary: bool = False


@dataclass
class ReviseInfo(TcBaseObj):
    """
    Information to identify the source item revision for the revise and attributes to apply to the new item revision.
    
    :var clientId: Identifier to uniquely identify the input.  Also used in the output to help identify which inputs
    had errors.
    :var baseItemRevision: Original item revision to be used for the 'revise' operation
    :var newRevId: Revision ID string for creating new revision
    :var name: Name to apply to the new revision.
    :var description: Description to apply to the new revision.
    :var deepCopyInfo: List of 'DeepCopyData' which represents rules to be used for propagating the relations of the
    ItemRevision object
    :var newItemRevisionMasterProperties: The properties to be set for the newly created master form of the new
    ItemRevision object.
    """
    clientId: str = ''
    baseItemRevision: ItemRevision = None
    newRevId: str = ''
    name: str = ''
    description: str = ''
    deepCopyInfo: List[DeepCopyData] = ()
    newItemRevisionMasterProperties: MasterFormPropertiesInfo = None


@dataclass
class ReviseOutput(TcBaseObj):
    """
    Output  structure for 'revise' operation.
    
    :var newItemRev: Newly created item revision.
    :var relatedObjects: A list of 'RelatedObjectInfo' which represents the new objects created or the existing objects
    used to propagate the relations during the deep copy.
    """
    newItemRev: ItemRevision = None
    relatedObjects: List[RelatedObjectInfo] = ()


@dataclass
class ReviseResponse(TcBaseObj):
    """
    Results of the service call. The output is a map of the input 'clientId' and the results of the input data
    associated with the 'clientId'.
    
    :var reviseOutputMap: Map whose keys are the input 'clientIds' and values are 'ReviseOutput' structures.
    :var serviceData: Standard ServiceData member that will contain the newly created Iitem Rrevision in the created
    objects list. The deep copy rules used, (either supplied by the client or, if not supplied, then the default deep
    copy rules), will create new objects for any copyAsObject rules and those objects will also be in the created
    objects list. For deep copy rules of copyAsReference, those objects will be in the plain objects list. 
    It will also contain any errors encountered. Any errors will also contain the clientId so the client can cross
    reference the error to the input data. Errors  generated by this service include :
    7007 Invalid Item revision tag
    215269 Failure creating new form.
    """
    reviseOutputMap: ReviseOutputMap = None
    serviceData: ServiceData = None


@dataclass
class SaveAsNewItemInfo(TcBaseObj):
    """
    The 'SaveAsNewItemInfo' structure is the main input to the 'saveAsNewItem' service. This structure refers to the
    item to be saved and information on how the new item is to be stored.
    
    :var clientId: Identifier to uniquely identify the input
    :var baseItemRevision: Original item revision to be used for the operation.
    :var itemPropertyInfo: Attributes to apply to the new item, a list of 'PropertyNameValueInfos'.
    :var newRevId: Revision ID for the new revision
    :var type: Type of the new item.
    :var deepCopyInfo: List of 'DeepCopyData's to be used for propagating the relations of the item revision
    :var newItemMasterProperties: The properties to be set for the newly created master form of the new item.
    :var newItemRevisionMasterProperties: The properties to be set for the newly created master form of the new item
    revision.
    :var containingFolder: The destination folder for the new item. If null, the item will go to the  folder specified
    in 'WsoInsertNoSelectionsPref' preference.
    """
    clientId: str = ''
    baseItemRevision: ItemRevision = None
    itemPropertyInfo: List[PropertyNameValueInfo] = ()
    newRevId: str = ''
    type: str = ''
    deepCopyInfo: List[DeepCopyData] = ()
    newItemMasterProperties: MasterFormPropertiesInfo = None
    newItemRevisionMasterProperties: MasterFormPropertiesInfo = None
    containingFolder: Folder = None


@dataclass
class SaveAsNewItemOutput(TcBaseObj):
    """
    The output structure for the 'SaveAsNewItem' service.
    
    :var newItem: The newly created item.
    :var newItemRev: The newly created item revision.
    :var relatedObjects: The list of Related Objects that were created or referenced when applying the deep copy rules.
    """
    newItem: Item = None
    newItemRev: ItemRevision = None
    relatedObjects: List[RelatedObjectInfo] = ()


@dataclass
class SaveAsNewItemResponse(TcBaseObj):
    """
    Return structure for 'saveAsNewItem' operation
    
    :var saveAsOutputMap: Map whose keys are the input 'clientId's and values are 'SaveAsNewItemOutput' structures
    :var serviceData: Standard 'ServiceData' member that will contain the newly created item revision in the created
    objects list. The deep copy rules used, either supplied by the client or, if not supplied, then the default deep
    copy rules, will create new objects for any copyAsObject rules and those objects will also be in the created
    objects list. For deep copy rules of copyAsReference, those objects will be in the plain objects list
    Will also contain any errors generated by the service of system utilities. Errors will contain the 'clientId' so
    the error can be associated with the input. Errors generated by the service itself include :
        7007 Invalid Item revision
      48043 Duplicate item id ( Item already exists )
    215009 Invalid Item type
    215262 For deep copy with copy other side object, the new name is empty.
    215265 Failure copying dataset.
    215275 Deep copy action not supported.
    215267 Failure copying form.
    215269 Failure creating new form.
    215121 Invalid folder specified.
    """
    saveAsOutputMap: SaveAsNewItemOutputMap = None
    serviceData: ServiceData = None


@dataclass
class DeepCopyData(TcBaseObj):
    """
    The 'DeepCopyData' data structure overrides the deep copy rules defined by the system. Only the data supplied will
    be used. If no data is supplied then all the system data will be used.
    
    :var otherSideObject: This is a reference on which the deep copy action needs to be performed. Must be a Dataset or
    a Form.
    :var nRdeepCopyInfo: Named reference deep copy info structure. Only used if 'otherSideObject' is a Dataset and
    CopyAsObject is specified.    
    :var relationTypeName: This is the relation that needs to be deep copied.
    :var newName: This is the new name for the copied object represented by the 'otherSideObject' member. The value for
    the 'newName' will be ignored if the 'action' is not CopyAsObject or ReviseAndRelateToLatest.
    :var action: This is an integer representing the action to be performed on the object represented by
    'otherSideObject' member. The values for action are: 
    CopyAsObjectType = 0
    CopyAsRefType = 1
    DontCopyType =2
    RelateToLatest = 3
    ReviseAndRelateToLatest = 4
    
    :var isTargetPrimary: Flag representing whether the given item revision is the primary object in the relation. This
    property is passed directly to the deep copy utility for the item revision.
    :var isRequired: Flag representing whether the deep copy information is from a mandatory deep copy rule configured
    by the system. If set then the copy is required to be performed and cannot be overridden. This property is passed
    directly to the deep copy utility for the item revision.
    :var copyRelations: Flag representing whether the relations are to be copied. If true then all relations on the
    current object are copied to the new object. If false then no relations are copied. This property is passed
    directly to the deep copy utility for the Item Revision.
    """
    otherSideObject: BusinessObject = None
    nRdeepCopyInfo: List[DeepCopyDataForNR] = ()
    relationTypeName: str = ''
    newName: str = ''
    action: int = 0
    isTargetPrimary: bool = False
    isRequired: bool = False
    copyRelations: bool = False


@dataclass
class DeepCopyDataForNR(TcBaseObj):
    """
    Deep copy data for named references. Only used if 'otherSideObject' is a dataset and the action is CopyAsObject.
    
    :var otherSideObject: This is a reference on which the deep copy action needs to be performed. Currently only
    Dataset and Form objects are supported.
    :var referenceName: This is the named reference name that needs to be deep copied.
    :var newName: This is the new name for the copied object represented by the 'otherSideObject' member.
    :var action: This is an integer representing the action to be performed on the object represented by the
    'otherSideObject' member. The values for action are: 
    CopyAsObjectType = 0
    CopyAsRefType = 1
    DontCopyType =2
    """
    otherSideObject: BusinessObject = None
    referenceName: str = ''
    newName: str = ''
    action: int = 0


"""
A map of the clientId from the input data to the output from the service.
"""
ReviseOutputMap = Dict[str, ReviseOutput]


"""
A map of the clientId from the input data to the output from the service.
"""
SaveAsNewItemOutputMap = Dict[str, SaveAsNewItemOutput]



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetODSVersionResponse(TcBaseObj):
    """
    'GetODSVersionResponse' structure contains ODS version information and Servicedata
    
    :var serverReleaseVersion: Major release version of Teamcenter server hosting the ODS.
    :var serverMinorReleaseVersion: Minor release version of Teamcenter server hosting the ODS.
    :var serverServiceReleaseVersion: Service release version of Teamcenter server hosting the ODS.
    :var serverMaintPackVersion: Maintenance pack version of the server hosting the ODS.
    :var serverOdsMajorVersion: ODS services major version.
    :var serverOdsMinorVersion: ODS services minor version.
    :var clientRejected: Access to the requesting TcEnt client rejected/ accepted.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    serverReleaseVersion: int = 0
    serverMinorReleaseVersion: int = 0
    serverServiceReleaseVersion: int = 0
    serverMaintPackVersion: int = 0
    serverOdsMajorVersion: int = 0
    serverOdsMinorVersion: int = 0
    clientRejected: bool = False
    serviceData: ServiceData = None


@dataclass
class QueryPublicationRecordsProperties(TcBaseObj):
    """
    'QueryPublicationRecordsProperties' structure holds the paramerters which needs to be queried with the ODS server.
    
    :var attributeName: Name of the attribute.
    :var attributeValue: Respective value for the attribute
    :var attributeType: Attribute type
    :var qryOperator: Standandard query operator.
    """
    attributeName: str = ''
    attributeValue: str = ''
    attributeType: str = ''
    qryOperator: str = ''


@dataclass
class QueryPublicationRecordsResponse(TcBaseObj):
    """
    'QueryPublicationRecordsResponse' structure holds the retrieved publication records.
    
    :var publishedObjTagAsString: List of PUIDs of the published object
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    publishedObjTagAsString: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class RepublishObjectInputProperties(TcBaseObj):
    """
    'RepublishObjectInputProperties'  structure contains list of property values which needs to be added in publication
    record.
    
    :var oldObjectTag: Unique id allocated to client site in the multisite federation
    :var pubRecProps: Object of 'TcEntPublicationRecordProperties' with the major attributes as object tag and owning
    site id.
    """
    oldObjectTag: str = ''
    pubRecProps: TcEntPublicationRecordProperties = None


@dataclass
class TcEntClientInfoProperties(TcBaseObj):
    """
    'TcEntClientInfoProperties' structure contains requesting ODS client information.
    
    :var clientSiteId: Unique id allocated to client TcEnt site.
    :var targetServerSiteId: ODS server site ID.
    :var clientNode: Machine name where TcEnterprise Server is running.
    :var clientReleaseVersion: TcEnterprise (ODS Client) server release version.
    :var clientServiceReleaseVersion: TcEnterprise (ODS Client) server release version.
    :var clientMaintPackVersion: TcEnterprise (ODS Client) server maintainace pack version.
    :var userName: Teamcenter user ID.
    :var groupName: Teamcenter group associated with the given user name.
    :var roleName: Teamcenter role associated with the given user name.
    :var customAttributeNames: The extra attributes added for the TcEnterprise (ODS Client) server client. This is used
    in case of extensible ODS feature.
    """
    clientSiteId: int = 0
    targetServerSiteId: int = 0
    clientNode: str = ''
    clientReleaseVersion: int = 0
    clientServiceReleaseVersion: int = 0
    clientMaintPackVersion: int = 0
    userName: str = ''
    groupName: str = ''
    roleName: str = ''
    customAttributeNames: MapData = None


@dataclass
class TcEntPublicationRecordProperties(TcBaseObj):
    """
    'TcEntPublicationRecordProperties' structure contains list of all PublicationRecord properties.
    
    :var owningSiteId: Unique id allocated to to client TcEnt site
    :var objectTag: Unique tag allocated to the object within session
    :var pubDate: Date of publication of Object
    :var creationDate: Date of creation of Object
    :var projectName: Project name, where object belongs
    :var releaseStatus: Object release status
    :var customAttributes: The extra attributes added for the TcEnterprise (ODS Client) server client. This is used in
    case of extensible ODS feature.
    :var objectId: Unique object id
    :var revisionId: Revision id of the published object
    :var objectName: Object name of published Object.
    :var objectDesc: Object Description if any, else the empty string
    :var objectClass: Classs of the Object
    :var objectType: Object type
    :var owningUserId: User id who owns the object
    :var owningGroupName: Groud name, where user belongs to
    """
    owningSiteId: int = 0
    objectTag: str = ''
    pubDate: datetime = None
    creationDate: datetime = None
    projectName: str = ''
    releaseStatus: List[str] = ()
    customAttributes: MapData = None
    objectId: str = ''
    revisionId: str = ''
    objectName: str = ''
    objectDesc: str = ''
    objectClass: str = ''
    objectType: str = ''
    owningUserId: str = ''
    owningGroupName: str = ''


@dataclass
class DescribePublicationRecordsResponse(TcBaseObj):
    """
    'DescribePublicationRecordsResponse' structure contains list of all PublicationRecord properties and 'Servicedata'.
    
    :var records: List of 'TcEntPublicationRecordProperties' object.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message
    """
    records: List[TcEntPublicationRecordProperties] = ()
    serviceData: ServiceData = None


"""
MapData
"""
MapData = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Discipline, POM_object, ScheduleTask, ResourcePool
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AssignmentCreateContainer(TcBaseObj):
    """
    AssignResource SOA
    
    :var task: ScheduleTask.
    :var resource: Resource.
    :var discipline: Discipline.
    :var assignedPercent: Percent
    :var placeholderAssignment: PlaceHolder assignment.
    :var isPlaceHolder: isPlaceHolder
    """
    task: ScheduleTask = None
    resource: POM_object = None
    discipline: Discipline = None
    assignedPercent: float = 0.0
    placeholderAssignment: ResourcePool = None
    isPlaceHolder: bool = False



from __future__ import annotations

from tcsoa.gen.Workflow._2013_05.Workflow import GetWorkflowTemplatesResponse, GetWorkflowTemplatesInputInfo
from typing import List
from tcsoa.utils import TcService


class WorkflowService(TcService):

    @classmethod
    def getWorkflowTemplates(cls, input: List[GetWorkflowTemplatesInputInfo]) -> GetWorkflowTemplatesResponse:
        """
        This operation gets the list of workflow templates based on input criteria. The criteria includes the target
        objects for the workflow or the types of the target objects. User can also specify if the under construction
        templates should to be returned and if the filtered list of templates is required.
        The filtered list of templates are returned based on the users group and the target objects. The filter rules
        can also be customized using the user exits.
        """
        return cls.execute_soa_method(
            method_name='getWorkflowTemplates',
            library='Workflow',
            service_date='2013_05',
            service_name='Workflow',
            params={'input': input},
            response_cls=GetWorkflowTemplatesResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ItemRevision, BOMLine, MEAppearancePathNode
from typing import Dict, List
from tcsoa.gen.Internal.Rdv._2009_01.VariantManagement import NVEMetaToken
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class NodeAnswer(TcBaseObj):
    """
    Structure containing list of valid background nodes for distance/target
    
    :var backgroundNodeList: List of AppearancePathNode UIDs
    """
    backgroundNodeList: List[str] = ()


@dataclass
class OccNotesAttributes(TcBaseObj):
    """
    The part data information about each component is provided by way of the OccNotesAttributes data structure.
    
    :var noteType: The Note Type for the Part Solution to be added.
    :var noteText: The Note Text for the Part Solution to be added.
    """
    noteType: str = ''
    noteText: str = ''


@dataclass
class ProximityTarget(TcBaseObj):
    """
    Structure contain references to proximity distance and target BOMLine objects APN UID.
    
    :var proxtargetId: Identifier that helps the client track the object(s) created.
    :var proximity: Proximity Distance in assembly units (supported by the underlying search engine)
    :var target: target BOMLine APN UID
    """
    proxtargetId: str = ''
    proximity: float = 0.0
    target: str = ''


@dataclass
class ReplacePartSolutionInputInfo(TcBaseObj):
    """
    The information to replace part is provided by way of the 'ReplacePartSolutionInputInfo' data structure.
    
    :var values: List of OccNotesAttributes objects for each replacing component that contains the Part data for the
    replacing Part Solution, i.e. all the Note Types for the Part Solution along with the corresponding Note Texts.
    :var component: The reference to the Part Solution to be replaced with.  It should be of type Item Revision or its
    subtypes. This is a mandatory attribute for the operation to succeed and cannot be null.
    :var bomLine: The BOMLine object for which the Part Solution is to be replaced. This is a mandatory attribute for
    the operation to succeed and cannot be null.
    :var abeAPN: The Appearance Path Node of the associated Architecture Breakdown Element.
    :var metaExpr: List of NVEMetaExpression objects to be applied to new replaced Part Solution. The Meta Expression
    helps in balancing out the variant condition before generating the variant condition.
    :var validate: A flag. If set to true the operation will validate the combination of variants supplied through the
    NVEs and will fail the operation if the validation fails.
    :var splitAndClone: A flag which states whether the clone should be created or not.
    :var carrySubstitutes: A flag which states whether the substitutes should be carried forward or not.
    :var quantity: The quantity of the replacing Part Solution.
    """
    values: List[OccNotesAttributes] = ()
    component: ItemRevision = None
    bomLine: BOMLine = None
    abeAPN: MEAppearancePathNode = None
    metaExpr: List[NVEMetaToken] = ()
    validate: bool = False
    splitAndClone: bool = False
    carrySubstitutes: bool = False
    quantity: int = 0


@dataclass
class ReplacePartSolutionOutputInfo(TcBaseObj):
    """
    The structure 'ReplacePartSolutionOutputInfo' represents the output information for 'replacePartInProduct'
    operation.
    
    :var bomLine: The bomline for which the Part Solution is to be replaced.
    :var component: The newly replaced Part Solution. It will be of type Item Revision or its subtypes.
    """
    bomLine: BOMLine = None
    component: ItemRevision = None


@dataclass
class ReplacePartSolutionResponse(TcBaseObj):
    """
    The 'ReplacePartSolutionResponse' structure represents the reference of the newly added component and the
    'ServiceData' object.
    
    :var output: The list of  ReplacePartSolutionOutputInfo objects.
    :var serviceData: The 'ServiceData' object containing error codes and error messages along with the indices for
    which the 'replacePartInProduct' operation fails.
    """
    output: List[ReplacePartSolutionOutputInfo] = ()
    serviceData: ServiceData = None


@dataclass
class BackgroundOccurrences(TcBaseObj):
    """
    Output structure that contains the references to valid background nodes
    
    :var numProcessedRequests: actual number of the answers (empty aswell) provided
    :var numSuggestedRequests: server suggested number of the answers provided. Ex:100
    :var nodeanswerLists: list of 'NodeAnswer' objects corresponding to the respective targets
    """
    numProcessedRequests: int = 0
    numSuggestedRequests: int = 0
    nodeanswerLists: List[NodeAnswer] = ()


@dataclass
class BackgroundOccurrencesResponse(TcBaseObj):
    """
    Main output structure that contains the references to valid background nodes (APN or Absocc) UIDs and 'ServiceData'
    
    :var output: List of 'BackgroundOccurrence' objects for respective 'TargetOccurence' objects
    :var servicedata: The SOA framework object containing objects that were created, deleted, or updated by the
    Service, plain objects, and error information.
    """
    output: BackgroundOccurrencesMap = None
    servicedata: ServiceData = None


@dataclass
class TargetOccurrences(TcBaseObj):
    """
    Main input structure containing references to distance and targets
    
    :var clientId: Identifier that helps the client track the object(s) created. This is used to correlate the input
    objects with the output response being sent back to the client.
    :var topitemrev: UID of top ItemRevision
    :var revrulename: Name of the RevisionRule
    :var disttargetsList: List of target APN or Absocc UIDs and corresponding proximity distances
    """
    clientId: str = ''
    topitemrev: str = ''
    revrulename: str = ''
    disttargetsList: List[ProximityTarget] = ()


@dataclass
class AddPartSolutionInputInfo(TcBaseObj):
    """
    The information of added part is provided by way of the 'AddPartSolutionInputInfo' data structure.
    
    :var values: The 'OccNotesAttributes' for each added component that contains the Part data for the Part Solution 
    to be added, i.e. all the Note Types for the Part Solution along with the corresponding Note Texts.
    :var component: The reference to the Part Solution to be added.  It should be of type Item Revision or its
    subtypes. This is a mandatory attribute for the operation to succeed and cannot be null.
    :var abeLine: The BOMLine object to which the Part Solution is to be added. This is a mandatory attribute for the
    operation to succeed and cannot be null.
    :var prodRevs: The reference to the Product Item Revisions associated with the Architecture Breakdown for each Part
    Solution to be added. This is a mandatory attribute for the operation to succeed and cannot be null.
    :var abeApn: The Appearance Path Node for each  Part Solution to be added.
    :var metaExpr: The list of NVEMetaExpression objects to be applied  to new Part Solution. The Meta Expression helps
    in balancing out the variant condition before generating the variant condition.
    :var validate: A flag if set to true the operation will validate the combination of variants supplied through the
    NVEs and will fail the operation if the validation fails.
    :var quantity: The quantity of the Part Solution to be added.
    """
    values: List[OccNotesAttributes] = ()
    component: ItemRevision = None
    abeLine: BOMLine = None
    prodRevs: List[ItemRevision] = ()
    abeApn: MEAppearancePathNode = None
    metaExpr: List[NVEMetaToken] = ()
    validate: bool = False
    quantity: int = 0


@dataclass
class AddPartSolutionOutputInfo(TcBaseObj):
    """
    The structure 'AddPartSolutionOutputInfo' represents the output information for 'addPartToProduct' operation.
    
    :var component: The Part Solution to be added. It should be of type Item Revision or its subtypes.
    :var partSolutionAPN: Appearance Path Node of the newly added Part Solution.
    """
    component: ItemRevision = None
    partSolutionAPN: MEAppearancePathNode = None


@dataclass
class AddPartSolutionResponse(TcBaseObj):
    """
    The 'AddPartSolutionResponse' structure represents reference to the Appearance Path Node corresponding to the newly
    added part solution and the 'ServiceData' object.
    
    :var output: List of  'AddPartSolutionOutputInfo' objects containing Appearance Path Nodes corresponding to each of
    the newly added Part Solutions.
    :var serviceData: The 'ServiceData' object containing error codes and error messages along with the indices for
    which the 'addPartToProduct' operation fails.
    """
    output: List[AddPartSolutionOutputInfo] = ()
    serviceData: ServiceData = None


"""
Mapping of clientId to output BackgroundOccurences
"""
BackgroundOccurrencesMap = Dict[str, BackgroundOccurrences]



from __future__ import annotations

from tcsoa.gen.Internal.Visualization._2010_09.DataManagement import CreateSnapshot3DResponse
from typing import List
from tcsoa.gen.Internal.Visualization._2012_10.DataManagement import NewSnapshot3DInput
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def createSnapshot3D(cls, input: List[NewSnapshot3DInput]) -> CreateSnapshot3DResponse:
        """
        This operation creates the Product View data model (or portions of it if desired) consisting of a
        SnapShotViewData Dataset and associated files and relationships.  The visible lines along with a BusinessObject
        where the Dataset is to be attached are sent as input to the operation. Product view Dataset creation
        information (name and description) should be sent as input to the operation as well. Two files are ultimately
        required by the data model (a viewFile (PLMXML) file, and a thumbnail image (jpg, cgm)).  Several optional
        files can also be specified (a preview image (*.*), motion (vfm), markup layers (vpl), and 3D Geometry Asset
        image (asset)).  These files can first be exported prior to calling this service, and the entire model will be
        created by this service.
        
        The server implementation of this operation will create the structure (PLMXML) file that represents an export
        of the structure for all visible lines and their descendants if specified.  However, creation of the structure
        (PLMXML) file can be very costly from a performance perspective.  As a result, the service has been set up so
        that the caller can initiate this call early in the save process just to create the basics of the model
        including export of structure file before gathering the required files from the viewer (i.e. Thumbnail (jpg,
        cgm), ViewFile (PLMXML), and the optional files) and uploading them.  If this approach is used, the viewer
        files must be saved later using the UpdateSnapshot3D operation.  The operation is set up to handle both
        approaches.
        
        In the case the input visible lines are BOMLines, the server implementation of this service creates a
        VisStructureContext object (or objects) representing the configuration of the loaded structure based on the
        BOPWindow or BOMWindow specified via the BOMLines passed for the visible lines. This object is capable of
        storing configuration information for Product and Process structures, including composition structures.  This
        allows clients to reopen a Product View with the exact same structure configuration that was in effect when the
        Product View was created for all the structure types supported by the BVR model.
        
        The service returns the map between client IDs and created Dataset objects which may or may not have all the
        necessary files attached depending on the calling implementation.
        
        Use cases:
        This operation is for clients that have loaded a 3D structure from Teamcenter in the
        Active Workspace 3D Viewer. The 'attach to' location is the object where the Product view stored is to be
        attached.'
        
        Currently there is one basic use case for calling this method:
        
        - Create dataset with view PLMXML, thumbnail and high resolution only, for Active Workspace View Captures.
        
        
        
        View Capture Generation:
        
        
        - Client application with visualization capabilities loads a Product structure and visualizes 3D geometry from
        Teamcenter.
        - Client gathers the list of visible Business Objects for the 3D scene.
        - Client gathers the attach to Object as the save location for the Product View.
        - Client gathers Dataset info (name, description, etc) from the user.
        - Client exports the following files locally that capture the view data: a viewFile (*.plmxml), a thumbnail
        image (*.jpg, *.cgm), and a preview image (*.*).  
        - From these files and knowledge of the model, the client build up the named reference information for the
        SnapShotViewData Dataset needed for uploading the files.
        - Client calls createSnapshot3D operation and passes a reference to Object to which snapshot Dataset is to be
        attached, relation to use (View Capture), datasetInfo, visibleLinesList, and named reference information
        - Teamcenter/Server creates the complete Product View data model per the inputs.
        
        """
        return cls.execute_soa_method(
            method_name='createSnapshot3D',
            library='Internal-Visualization',
            service_date='2012_10',
            service_name='DataManagement',
            params={'input': input},
            response_cls=CreateSnapshot3DResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.Notification._2015_03.SubscriptionManagement import GetEventTypesResponse, SubscriptionInput2, GetSubscriptionInputResponse, FindSubscriptionsInput2, TransferNotificationInput, GetSubscribableTypesResponse, ModifySubscriptionsInput2, EventObject, GetAttachEventDataResponse, GetAssignEventDataResponse
from typing import List
from tcsoa.gen.Notification._2014_10.SubscriptionManagement import SubscriptionsResponse
from tcsoa.utils import TcService
from tcsoa.gen.Internal.Notification._2015_03.MessageManagement import UserGroupAliasResponse


class SubscriptionManagementService(TcService):

    @classmethod
    def getSubscribableTypes(cls, childTypeOption: str) -> GetSubscribableTypesResponse:
        """
        This operation retrieves types and subtypes of business objects for which notification subscriptions can be
        created. The  types and subtype objects are based on the childTypeOption.
        
        Use cases:
        The client creates a Subscription in order to trigger functionality when a specific event occurs against the
        object. Subscriptions cannot be created for all types of objects. There are only a certain types of objects
        that are subscribable. These subscribable object types are returned.
        """
        return cls.execute_soa_method(
            method_name='getSubscribableTypes',
            library='Internal-Notification',
            service_date='2015_03',
            service_name='SubscriptionManagement',
            params={'childTypeOption': childTypeOption},
            response_cls=GetSubscribableTypesResponse,
        )

    @classmethod
    def getSubscriptionInput(cls, input: BusinessObject) -> GetSubscriptionInputResponse:
        """
        The getSubscriptionInput operation retrieves the valid subscribable events the business object in the input can
        have. When an event is subscribable, that means subscriptions can be created for that event. A list of action
        handlers and attribute criteria is also retrieved. These events and handlers are populated after selection of
        an object that is to be subscribed.
        
        Use cases:
        The client selectes a subscribable business object to be able to create subscriptions on it. The client is
        returned with a list of event types and action handlers applicable on that business object. If there is an
        attribute criteria configuration file specified in the TC_DATA folder, the attribute criteria will be populated
        in the response for the business object type.
        """
        return cls.execute_soa_method(
            method_name='getSubscriptionInput',
            library='Internal-Notification',
            service_date='2015_03',
            service_name='SubscriptionManagement',
            params={'input': input},
            response_cls=GetSubscriptionInputResponse,
        )

    @classmethod
    def modifySubscriptions(cls, inputs: List[ModifySubscriptionsInput2]) -> SubscriptionsResponse:
        """
        This operation modifies existing ImanSubscription objects.  All values specified in the inputs replace the
        values on the ImanSubscription in the database.  Each modification is verified to insure that a duplicate
        ImanSubscription is not created.
        
        Use cases:
        An existing Subscription needs to have one or more of its values modified.
        """
        return cls.execute_soa_method(
            method_name='modifySubscriptions',
            library='Internal-Notification',
            service_date='2015_03',
            service_name='SubscriptionManagement',
            params={'inputs': inputs},
            response_cls=SubscriptionsResponse,
        )

    @classmethod
    def transferNotifications(cls, input: TransferNotificationInput) -> SubscriptionsResponse:
        """
        This operation transfers the ownership of a subscription The transfer can be temporary (for a selected date
        range) or permanent.
        """
        return cls.execute_soa_method(
            method_name='transferNotifications',
            library='Internal-Notification',
            service_date='2015_03',
            service_name='SubscriptionManagement',
            params={'input': input},
            response_cls=SubscriptionsResponse,
        )

    @classmethod
    def createSubscriptions(cls, inputs: List[SubscriptionInput2]) -> SubscriptionsResponse:
        """
        This operation creates multiple object or class based Subscriptions based on the provided inputs. A
        Subscription allows a specified piece of functionality to be executed whenever an event occurs on the target
        object. Inputs to this operation will allow the target object to be specified, along with the event to monitor.
        The functionality the execute is specified by an array of ImanActionHandler objects. These handlers will be
        executed in the order they are specified in the Subscription definition. The most common handler would be
        IMAN_Smtp_Mail_Notify, which is used to send an e-mail when the event is triggered. If this handler is
        specified, the notification object should also be included as input.
        
        This operation also creates class based Subscriptions. This is done by passing a type object as the target
        object.  Class based Subscriptions will apply to all objects of the specified type. These can be filtered down
        at execution time by specifying a condition or attribute criteria as input. The Condition or attribute criteria
        will be evaluated against the object on which the event occurred, and the handler will only be executed if the
        Condition evaluates to TRUE.  Conditions and attribute criteria only apply for class based Subscriptions. 
        Attribute criteria are made up of a list of attributes and the values to compare them against off the target
        object.  Each of these comparisons can be logically combined.
        
        
        Use cases:
        Object Based Subscription:
        The client creates an object based Subscription in order to trigger functionality when a specific event occurs
        against the object. An example of this would be subscribing to the "Check Out" event for a given object. Any
        time the object is checked out, the handler functionality will be executed. Most often, this functionality will
        send the given user an e-mail notification.
        
        Class Based Subscription:
        The client creates a class based Subscription in order to trigger functionality when a specific event occurs
        against any object of the given type. An example of this would be subscribing to the "Check Out" event for an
        ItemRevision objects. Any time the object is checked out, the handler functionality will be executed. Most
        often, this functionality will send the given user an e-mail notification.
        
        Class based subscriptions allow a Condition or attribute criteria to be specified as part of the definition.
        This allows the events to be filtered to a smaller level. Any time the event occurs against the given object of
        the specified type, the Condition will be evaluated against the object. If the Condition or attribute criteria
        evaluate to true, the event is posted and the handlers are executed. An example Condition would only evaluate
        to true if the object is part of a specific Project.  Attribute criteria can also be specified and will be
        evaluated when the Subscription is being processed.  If the attribute criteria values do not match, the
        Subscription will be rejected.  Both attribute criteria and a Condition can not be specified at the same time.
        """
        return cls.execute_soa_method(
            method_name='createSubscriptions',
            library='Internal-Notification',
            service_date='2015_03',
            service_name='SubscriptionManagement',
            params={'inputs': inputs},
            response_cls=SubscriptionsResponse,
        )

    @classmethod
    def findSubscriptions(cls, input: FindSubscriptionsInput2) -> SubscriptionsResponse:
        """
        This operation queries for all Subscriptions that meet the input criteria. A variety of input parameters are
        allowed, including target, event type, subscriber, condition, handlers, execution time, expiration date,
        notification priority, status and temporary transfer information.
        
        Use cases:
        The client is searching for a previously created Subscription against a given object or type.
        """
        return cls.execute_soa_method(
            method_name='findSubscriptions',
            library='Internal-Notification',
            service_date='2015_03',
            service_name='SubscriptionManagement',
            params={'input': input},
            response_cls=SubscriptionsResponse,
        )

    @classmethod
    def getAssignEventData(cls) -> GetAssignEventDataResponse:
        """
        Description:
        The getAssignEventData operation retrieves the valid TaskType objects related to the assign event. When an
        event is subscribable, that means subscriptions can be created for that event. The attribute criteria for the
        subscription can be populated on the basis of the data returned using this operation.
        
        Use cases:
        The client selects a subscribable business object to be able to create subscriptions on it. The client is
        returned with a list of event types using the getSubscriptionInput operation. Then, if the user selects assign
        event, the list of TaskType objects is returned.
        """
        return cls.execute_soa_method(
            method_name='getAssignEventData',
            library='Internal-Notification',
            service_date='2015_03',
            service_name='SubscriptionManagement',
            params={},
            response_cls=GetAssignEventDataResponse,
        )

    @classmethod
    def getAttachEventData(cls, target: BusinessObject) -> GetAttachEventDataResponse:
        """
        The getAttachEventData operation retrieves the lists of ImanRelation objects and attachment types applicable on
        the target object. When an event is subscribable, that means subscriptions can be created for that event. The
        attribute criteria for the subscription can be populated on the basis of the data returned using this operation.
        
        Use cases:
        The client selects a subscribable business object that will be the target object of a subscription. The client
        is returned with a list of event types using the getSubscriptionInput operation. Then, if the user selects
        attach event, the lists of ImanRelation objects and attachment types applicable on the target object are
        returned. 
        """
        return cls.execute_soa_method(
            method_name='getAttachEventData',
            library='Internal-Notification',
            service_date='2015_03',
            service_name='SubscriptionManagement',
            params={'target': target},
            response_cls=GetAttachEventDataResponse,
        )

    @classmethod
    def getEventTypes2(cls, input: List[EventObject]) -> GetEventTypesResponse:
        """
        The operation retrieves the valid auditable and subscribable events for each of the business objects in the
        input EventObject list. When an event is auditable, you can audit actions on Teamcenter objects when that event
        happens on the business object. When an event is subscribable, subscriptions can be created for that event.
        
        Use cases:
        The client selects a subscribable or an auditable business object. Upon selection, a list of applicable event
        types is returned.
        """
        return cls.execute_soa_method(
            method_name='getEventTypes2',
            library='Internal-Notification',
            service_date='2015_03',
            service_name='SubscriptionManagement',
            params={'input': input},
            response_cls=GetEventTypesResponse,
        )


class MessageManagementService(TcService):

    @classmethod
    def getUserGroupAlias(cls) -> UserGroupAliasResponse:
        """
        This operation retrieves the list of all Teamcenter User, Group, and ImanAliasList objects.
        
        Use cases:
        The client needs a list of the users, groups or addresses to select from, for the notification of an event on
        an object. The user can then select from the populated lists of user, group or alias list objects to send the
        notification to.
        """
        return cls.execute_soa_method(
            method_name='getUserGroupAlias',
            library='Internal-Notification',
            service_date='2015_03',
            service_name='MessageManagement',
            params={},
            response_cls=UserGroupAliasResponse,
        )



from tcsoa.gen.Internal_GlobalMultiSite._2020_01.services import BriefcaseService as imp0
from tcsoa.gen.Internal_GlobalMultiSite._2018_11.services import BriefcaseService as imp1
from tcsoa.gen.Internal_GlobalMultiSite._2007_06.services import SynchronizerService as imp2
from tcsoa.gen.Internal_GlobalMultiSite._2007_06.services import ImportExportService as imp3
from tcsoa.gen.Internal_GlobalMultiSite._2020_01.services import OwnershipRecoveryService as imp4
from tcsoa.gen.Internal_GlobalMultiSite._2008_06.services import SynchronizerService as imp5
from tcsoa.gen.Internal_GlobalMultiSite._2010_02.services import LowlevelOwnershipTransferService as imp6
from tcsoa.gen.Internal_GlobalMultiSite._2010_09.services import LowlevelOwnershipTransferService as imp7
from tcsoa.gen.Internal_GlobalMultiSite._2007_06.services import DataMigrationService as imp8
from tcsoa.gen.Internal_GlobalMultiSite._2007_06.services import BriefcaseService as imp9
from tcsoa.gen.Internal_GlobalMultiSite._2017_05.services import ImportExportService as imp10
from tcsoa.utils import TcService


class BriefcaseService(TcService):
    addMarkOTForCurrentUser = imp0.addMarkOTForCurrentUser
    checkBriefcaseLicense = imp1.checkBriefcaseLicense
    getBriefcasePreviewData = imp1.getBriefcasePreviewData
    getObjectsLockInfo = imp0.getObjectsLockInfo
    packageBriefcaseContents = imp9.packageBriefcaseContents
    queryMarkOT = imp0.queryMarkOT
    removeMarkOTForCurrentUser = imp0.removeMarkOTForCurrentUser
    unpackBriefcaseContents = imp9.unpackBriefcaseContents


class SynchronizerService(TcService):
    checkReplicaSyncState = imp2.checkReplicaSyncState
    createExportRecordOnStubReplication = imp2.createExportRecordOnStubReplication
    getCandidatesForClasseswOpts = imp5.getCandidatesForClasseswOpts
    getCandidatesForObjectswOpts = imp5.getCandidatesForObjectswOpts
    getCandidatesToSynchronizeForListOfClasses = imp2.getCandidatesToSynchronizeForListOfClasses
    getCandidatesToSynchronizeForListOfObjects = imp2.getCandidatesToSynchronizeForListOfObjects
    getExportedObjects = imp5.getExportedObjects
    updateMasterObjectsOnReplicaDeletion = imp2.updateMasterObjectsOnReplicaDeletion
    updateObjectsOnOwnershipChange = imp2.updateObjectsOnOwnershipChange


class ImportExportService(TcService):
    confirmExport = imp3.confirmExport
    dryRunExport = imp3.dryRunExport
    exportObjects = imp3.exportObjects
    importObjects = imp3.importObjects
    requestExportToRemoteSites = imp3.requestExportToRemoteSites
    requestImportFromRemoteSites = imp3.requestImportFromRemoteSites
    transformData = imp10.transformData


class OwnershipRecoveryService(TcService):
    deleteOtTransaction = imp4.deleteOtTransaction
    findOtTransactions = imp4.findOtTransactions
    recoverOwnership = imp4.recoverOwnership
    recoverOwnershipUsingBriefcase = imp4.recoverOwnershipUsingBriefcase


class LowlevelOwnershipTransferService(TcService):
    getObjectsForOwnershipTransfer = imp6.getObjectsForOwnershipTransfer
    getObjectsForOwnershipTransfer2 = imp7.getObjectsForOwnershipTransfer
    transferOwnership = imp6.transferOwnership
    updateOwnershipTransfer = imp6.updateOwnershipTransfer


class DataMigrationService(TcService):
    getRUO = imp8.getRUO
    setRUO = imp8.setRUO



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Ai._2013_05.Ai import FindRequestOnAiWithReferencesResponse
from typing import List
from tcsoa.utils import TcService


class AiService(TcService):

    @classmethod
    def findRequestOnAiWithReferences(cls, baseRef: List[BusinessObject], requestType: str) -> FindRequestOnAiWithReferencesResponse:
        """
        The operation queries for the latest RequestObjects (by creation date and type) on the latest
        ApplicationInterface Object ( by creation date) that references the input object in the base_refs member.
        Additional filtering based on type of RequestObject is also possible.
        """
        return cls.execute_soa_method(
            method_name='findRequestOnAiWithReferences',
            library='Ai',
            service_date='2013_05',
            service_name='Ai',
            params={'baseRef': baseRef, 'requestType': requestType},
            response_cls=FindRequestOnAiWithReferencesResponse,
        )



from __future__ import annotations

from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class PerformanceJournalLevelResponse(TcBaseObj):
    """
    Return structure for the current level of performance journaling.
    
    :var level: 0 - off, normal journaling
    1 - version 1 performance journaling
    2 - version 2 performance journaling
    """
    level: int = 0



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ItemRevision, Vendor, ImanRelation, Item
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ChangeVendorInputInfo(TcBaseObj):
    """
    Input structure containing Vendor to change, VendorPart to change and destination new Vendor.
    
    :var vendorToChange: This is Object pointing to the Vendor to change. If this input is specified, all the related
    VendorPart objects are processed for change Vendor operation. This input takes precedence over VendorPart objects
    input.
    :var vendorParts: VendorPart objects to be processed for change Vendor operation.
    :var newVendor: The new Vendor to be attached to the part.
    """
    vendorToChange: Vendor = None
    vendorParts: List[Item] = ()
    newVendor: Vendor = None


@dataclass
class GetVendorPartsWithSelRuleInputInfo(TcBaseObj):
    """
    A structure of CommercialPartRevision and condition name.
    
    :var comPartRev: Selected CommercialPartRevision object from BOM. This revision is used for searching the related
    VendorPart objects based on selection rule.
    :var conditionName: The name of the condition in Teamcenter which is used as a selection rule while finding the
    VendorPart objects. The service operation will try and search for a valid selection rule based on the condition
    name. If this input is given as an empty string, the default selection rule is used which is governed by the
    preference VMS_vendor_part_selection_rule.
    """
    comPartRev: ItemRevision = None
    conditionName: str = ''


@dataclass
class GetVendorPartsWithSelRuleResponse(TcBaseObj):
    """
    The response structure containing 'VendorPartData' structure which in turn stores information about found
    VendorPart objects.
    
    :var partData: The output structure, which is essentially a list of 'VendorPartData' structures returned. Each of
    them will contain relation and part information for each VendorPart found.
    :var serviceData: Standard Teamcenter service response data.
    """
    partData: List[VendorPartData] = ()
    serviceData: ServiceData = None


@dataclass
class ChangeVendorResponse(TcBaseObj):
    """
    Response structure for the 'changeVendor' operation contains statuses member. It is a list of 'ChangeVendorStatus'
    structures.
    
    :var statuses: List of 'ChangeVendorStatus' structures. There is an entry per entry in input structure.
    :var serviceData: Standard 'ServiceData' structure. It contains partial errors and failures along with the
    clientIds.
    """
    statuses: List[ChangeVendorStatus] = ()
    serviceData: ServiceData = None


@dataclass
class VendorPartAndRel(TcBaseObj):
    """
    Structure containing a VendorPart object and its relation object to a CommercialPart.
    
    :var vendorPart: VendorPart object.
    :var vmRepresentRel: Each of these VendorPart objects are related to the input CommercialPartRevision with a
    relation called VMRepresents. This object represents an instance of that relation.
    """
    vendorPart: Item = None
    vmRepresentRel: ImanRelation = None


@dataclass
class VendorPartData(TcBaseObj):
    """
    Structure containing a filtered list of relation and part information for each VendorPart objects.
    
    :var partAndRel: It is the vector of 'VendorPartAndRel' objects holding the information of individual VendorPart
    objects and relation of input CommercialPartRevision with each of them.
    """
    partAndRel: List[VendorPartAndRel] = ()


@dataclass
class ChangeVendorStatus(TcBaseObj):
    """
    This is the structure containing a list of structures for each VendorPart statuses.
    
    :var changedStatus: This is the structure containing a list of statuses for change Vendor operation.
    """
    changedStatus: List[ChangedPartStatus] = ()


@dataclass
class ChangedPartStatus(TcBaseObj):
    """
    This structure contains the status of VendorPart objects and remarks from the operation.
    
    :var oldPartStr: Id of old VendorPart.
    :var newPartStr: Id of new VendorPart created.
    :var notes: Remarks (notes) corresponding to the each change Vendor operation. e.g. Success, Failure.
    """
    oldPartStr: str = ''
    newPartStr: str = ''
    notes: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import TaskDependency, TCCalendar, Schedule, BusinessObject, ResourceAssignment, ScheduleTask, TCCalendarEvent
from tcsoa.gen.Internal.ProjectManagement._2007_06.ScheduleManagement import StringValContainer, KeyValContainer
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GenericAttributesContainer(TcBaseObj):
    """
    A collection of KeyValContainers or StringValContainers reprenting the create or updates for a single object,
    such as Schedule, ScheduleTask, TaskDependency, or ResourceAssignment.
    
    :var object: The tag to the object being updated.
    :var keyValContainer: A collection of updates.
    :var stringValContainer: A collection of updates.
    :var typedAttributesContainer: A collection of updates.
    """
    object: BusinessObject = None
    keyValContainer: List[KeyValContainer] = ()
    stringValContainer: List[StringValContainer] = ()
    typedAttributesContainer: List[TypedAttributesContainer] = ()


@dataclass
class MasterMetaData(TcBaseObj):
    """
    A container for the SubMasterMetaData.
    
    :var uid: The uid of a master schedule.
    :var start: The Start Date of that master schedule.
    :var finish: The Finish Date of that master schedule.
    """
    uid: str = ''
    start: datetime = None
    finish: datetime = None


@dataclass
class MultipleScheduleLoadResponses(TcBaseObj):
    """
    A container for the load schedule response
    
    :var serviceData: The ServiceData.
    :var scheduleData: A collection of the schedule load responses.
    """
    serviceData: ServiceData = None
    scheduleData: List[ScheduleLoadResponse] = ()


@dataclass
class ScheduleLoadResponse(TcBaseObj):
    """
    A container for the load schedule response.
    
    :var schedule: The schedule.
    :var scheduleTask: All the tasks in this schedule.
    :var resourceAssignment: All the resource assignments in this schedule.
    :var taskDependency: All the task dependencies in this schedule.
    :var calendar: All the calendars referenced in this schedule.
    :var calendarEvent: All the calendar events referenced in the calendars.
    :var submasterdata: The meta data regarding master schedules.
    """
    schedule: Schedule = None
    scheduleTask: List[ScheduleTask] = ()
    resourceAssignment: List[ResourceAssignment] = ()
    taskDependency: List[TaskDependency] = ()
    calendar: List[TCCalendar] = ()
    calendarEvent: List[TCCalendarEvent] = ()
    submasterdata: List[SubMasterMetaData] = ()


@dataclass
class ScheduleModifyContainer(TcBaseObj):
    """
    The information necessary to update a schedule.  It contains information about new, updated, and deleted tasks,
    resource assignments, and task dependencies.
    
    :var scheduleUpdates: Updates to the schedule.
    :var deleteAllTasks: Should it delete all the tasks before beginning?  If this is set, it over-rides everything.
    :var deletedAssignments: A list of resource assignments to delete.
    :var newTasks: The information needed to create new tasks.
    :var newDependencies: The information needed to create new dependencies.
    :var newAssignments: The information needed to create new resource assignments.
    :var taskUpdates: The information needed to update existing tasks.
    :var dependencyUpdates: The information needed to update existing dependencies.
    :var assignmentUpdates: The information needed to update existing resource assignments.
    :var deletedTasks: A list of tasks to delete.
    :var deletedDependencies: A list of dependencies to delete.
    """
    scheduleUpdates: GenericAttributesContainer = None
    deleteAllTasks: bool = False
    deletedAssignments: List[ResourceAssignment] = ()
    newTasks: List[GenericAttributesContainer] = ()
    newDependencies: List[GenericAttributesContainer] = ()
    newAssignments: List[GenericAttributesContainer] = ()
    taskUpdates: List[GenericAttributesContainer] = ()
    dependencyUpdates: List[GenericAttributesContainer] = ()
    assignmentUpdates: List[GenericAttributesContainer] = ()
    deletedTasks: List[ScheduleTask] = ()
    deletedDependencies: List[TaskDependency] = ()


@dataclass
class ScheduleModifyResponse(TcBaseObj):
    """
    The response of the schedule modification.
    
    :var newTasks: A list of new tasks.
    :var newDependencies: A list of new dependencies.
    :var newAssignments: A list of new assignments.
    """
    newTasks: List[ScheduleTask] = ()
    newDependencies: List[TaskDependency] = ()
    newAssignments: List[ResourceAssignment] = ()


@dataclass
class ScheduleModifyResponses(TcBaseObj):
    """
    A collection of responses.
    
    :var serviceData: The ServiceData.
    :var responses: A collection of reponses.
    """
    serviceData: ServiceData = None
    responses: List[ScheduleModifyResponse] = ()


@dataclass
class SubMasterMetaData(TcBaseObj):
    """
    A container for the SubMasterMetaData.
    
    :var subschedule: The tag of the sub schedule.
    :var masterdata: A collection of information about all master schedules of the subschedule.
    """
    subschedule: Schedule = None
    masterdata: List[MasterMetaData] = ()


@dataclass
class TypedAttributesContainer(TcBaseObj):
    """
    A container which is used to update custom properties.
    
    :var type: The object type.
    :var attributes: A collection of updates.
    """
    type: str = ''
    attributes: List[StringValContainer] = ()



from __future__ import annotations

from tcsoa.gen.Internal.AWS2._2017_06.Finder import ColumnConfig2
from tcsoa.gen.Internal.AWS2._2018_12.Finder import ExportSearchResultsResponse
from tcsoa.gen.Internal.AWS2._2018_12.MultiSite import FetchODSResponse, StringVectorMap
from typing import List
from tcsoa.gen.Internal.AWS2._2018_05.Finder import SearchInput2
from tcsoa.utils import TcService


class FinderService(TcService):

    @classmethod
    def exportSearchResults(cls, searchInput: SearchInput2, columnConfig: ColumnConfig2, exportAll: bool, selectedObjectUIDs: List[str], applicationFormat: str) -> ExportSearchResultsResponse:
        """
        This operation exports Teamcenter business objects based on search results to the MSExcelX application. The
        operation allows caller to specify a list of objects to export or to specify if they want to export all objects
        from the search result.
        If the caller chooses to export a list of objects then the object UIDs must be passed along with column
        configuration information. The order of the objects received will be the order of objects exported. 
        If the caller chooses to export all objects, then this operation requires searchInput object parameter to be
        populated which is used to perform a search and the result will be exported. The searchSortCriteria property
        will be used to sort the exported objects.
        
        Use cases:
        Use Case:
        1) The user selects single or multiple objects in Active Workspace client. The user clicks on the Export button
        and selects the option to export the selected objects and clicks on export, then the selected objects are
        exported to MsExcelX. 
        2) The user does not select any objects from list of search results in Active Workspace client. The user clicks
        on the Export button and selects the option to export all search results and clicks on export, then all objects
        are exported to MsExcelX.
        """
        return cls.execute_soa_method(
            method_name='exportSearchResults',
            library='Internal-AWS2',
            service_date='2018_12',
            service_name='Finder',
            params={'searchInput': searchInput, 'columnConfig': columnConfig, 'exportAll': exportAll, 'selectedObjectUIDs': selectedObjectUIDs, 'applicationFormat': applicationFormat},
            response_cls=ExportSearchResultsResponse,
        )


class MultiSiteService(TcService):

    @classmethod
    def fetchODSRecords(cls, odsSiteName: str, searchMode: int, startDate: str, endDate: str, pubrList: List[str], odsSessionInfo: StringVectorMap) -> FetchODSResponse:
        """
        This operation queries for Object Directory Service (ODS) publication records at the target ODS site based on
        the input criteria and serializes the data into one or more TC XML files. The search criteria can be date based
        or a list of objects which have been previously published. Read tickets are returned for the generated TC XML
        files.
        
        Use cases:
        This will be used to extract ODS data for indexing purposes. The output will be used to populate data to a
        search engine like Solr.
        """
        return cls.execute_soa_method(
            method_name='fetchODSRecords',
            library='Internal-AWS2',
            service_date='2018_12',
            service_name='MultiSite',
            params={'odsSiteName': odsSiteName, 'searchMode': searchMode, 'startDate': startDate, 'endDate': endDate, 'pubrList': pubrList, 'odsSessionInfo': odsSessionInfo},
            response_cls=FetchODSResponse,
        )



from __future__ import annotations

from tcsoa.gen.Ai._2006_03.Ai import Configuration
from tcsoa.gen.Ai._2008_05.Ai import GenerateArrangementsResponse
from tcsoa.gen.BusinessObjects import ItemRevision
from tcsoa.utils import TcService


class AiService(TcService):

    @classmethod
    def generateArrangements(cls, itemRev: ItemRevision, revRule: Configuration, bvType: str) -> GenerateArrangementsResponse:
        """
        The generateArrangements operation will generate a PLMXML file with arrangements for an Item Revision.
        This operation will find out the BOMView Revision with input BOMView Type associated to the Item Revision
        at first and then do generating process. An Item Revision and a BOMView Type can specify an unique BOMView
        Revision.
        If input BOMView Type is NULL, the default BOMView Type will be picked up. A revision rule can be applied to
        the BOMView Revision while generating. The output is struct GenerateArrangementsResponse with generated PLMXML
        file ticket
        and soa service data.
        """
        return cls.execute_soa_method(
            method_name='generateArrangements',
            library='Ai',
            service_date='2008_05',
            service_name='Ai',
            params={'itemRev': itemRev, 'revRule': revRule, 'bvType': bvType},
            response_cls=GenerateArrangementsResponse,
        )



from __future__ import annotations

from tcsoa.gen.Cad._2008_06.DataManagement import AttributeInfo
from tcsoa.gen.BusinessObjects import Dataset, POM_object, Fnd0ModelViewPalette, Fnd0ModelViewProxy, BusinessObject, WorkspaceObject, ImanFile
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GroupInfo(TcBaseObj):
    """
    A Model View Group to create or update and what Model View Proxies should be disclosed for that Group. It also
    allows each Group to be sequenced among the other Groups in the Palette.
    
    :var clientId: The use of this value is optional, but recommended.  It is used for mapping server errors and for
    correlating data in the response to the inputs.  The caller should populate a 'clientId' for each 'GroupInfo' with
    a value that is unique within the entire operation inputs.   The value is not interpreted or manipulated internally
    by the server
    :var boType: The type of group to create. Optional. If provided, must be a "Fnd0ModelViewGroup" or a valid sub-type
    of Fnd0ModelViewGroup. If not provided, "Fnd0ModelViewGroup" will be assumed as the type of the group to create.
    :var modelViewGroupForUpdate: Existing Model View Group (Fnd0ModelViewGroup or sub-type only) to be updated. If not
    given, a new Model View Group will be created and linked to the Model View Palette which was given as a Palette to
    create or update in the parent 'ModelViewPaletteInfo' input.
    :var attrListForGroup: A list of name-value pairs (string, list of strings) used to specify additional property
    data for the Model View Group.  All values are specified as strings, the caller is responsible for generating the
    correct string representation of each value passed.  For business objects use the UID value of the object.
    :var proxiesInGroup: An ordered list of Model View Proxies for the Group to contain. This is a complete list for
    this Model View Group.
    :var newOrderSequenceNumber: Optional hint for ordering Groups when not all Groups of a Palette are given. A 0
    'newOrderSequenceNumber' means to leave the Group in its current sequence order. Other valid sequence values are 1
    through the new total number of Group objects (e.g. the number of Groups after all Group creates and deletes for
    this Palette are completed.) If 'newOrderSequenceNumber' is 0 and this Group is newly created then it will be
    placed at the end of the existing Groups. No particular order is enforced among such zero-sequenced newly created
    Groups.
    """
    clientId: str = ''
    boType: str = ''
    modelViewGroupForUpdate: POM_object = None
    attrListForGroup: List[AttributeInfo] = ()
    proxiesInGroup: List[Fnd0ModelViewProxy] = ()
    newOrderSequenceNumber: List[int] = ()


@dataclass
class ModelViewPaletteInfo(TcBaseObj):
    """
    'ModelViewPaletteInfo' is used as input for creating, updating or deleting Model View Palette
    (Fnd0ModelViewPalette) and Model View Group (Fnd0ModelViewGroup) objects.  During update, 'boType' is left empty
    and the Model View Palette to update is given. Create or delete of a Model View Palette requires the user to have
    write access to the given 'disclosure'. Update of an existing Palette will only check write permission of the
    Palette itself.
    
    :var clientId: The use of this value is optional, but recommended.  It is used for mapping server errors and for
    correlating data in the response to the inputs.  The caller should populate a 'clientId' for each
    'ModelViewPaletteInfo' with a value that is unique within the input set and all input 'GroupInfo' for any input
    'ModelViewPaletteInfo'. The value is not interpreted or manipulated internally by the server.
    :var disclosure: The disclosure object (ItemRevision or Cpd0Workset) for which a Model View Palette is to be
    created, updated, or deleted. A value is optional. If not provided, any new Model View Palettes created will not be
    linked to any disclosure.
    :var boType: The business object type of a Model View Palette to be created. This value is optional for creating a
    new Model View Palette, and if provided must be Fnd0ModelViewPalette or a valid subtype of Fnd0ModelViewPalette. If
    no value is provided and no 'paletteToUpdate' is provided, then the 'boType' default value used is,
    "Fnd0ModelViewPalette." To update an existing Model View Palette, this value may be empty.
    :var paletteToUpdate: Required if an existing Palette (Fnd0ModelViewPalette) is being updated.
    :var attrListForPalette: A list of name-value pairs (string, list of strings) used to specify additional property
    data for the Model View Palette.  All values are specified as strings, the caller is responsible for generating the
    correct string representation of each value passed.  For business object, use the UID value of the object.
    :var groupsAndProxies: Details for new and existing Model View Groups (Fnd0ModelViewGroup) and their ordering and
    content.
    :var paletteIsComplete: If true, the existing list of Group objects (Fnd0ModelViewGroup) will be entirely replaced
    by the input Groups (within 'groupsAndProxies') and existing Groups in the Palette that are not in the
    'groupsAndProxies' content will be deleted (but not their Proxies). And if true, 'groupsToDelete' is ignored. If
    false, groups not in the 'groupsAndProxies' content are ignored, barring side-effects of requested re-ordering of
    Groups (see 'newOrderSequenceNumber' in 'groupsAndProxies').
    :var groupsToDelete: A list of Fnd0ModelViewGroup objects to delete and remove from the complete Palette
    (optional). List is ignored if 'paletteIsComplete' is true.
    :var deletePalette: If true, specifies that the existing Model View Palette (Fnd0ModelViewPalette) in
    'paletteToUpdate' should be deleted if possible. For update of an existing Model View Palette or creation of a new
    Palette, this value should be left at a default of false. All Model View Groups (Fnd0ModelViewGroup) objects in the
    deleted Palette will also be deleted but Proxies (Fnd0ModelViewProxy) referenced by the Model View Groups will be
    untouched.
    """
    clientId: str = ''
    disclosure: WorkspaceObject = None
    boType: str = ''
    paletteToUpdate: Fnd0ModelViewPalette = None
    attrListForPalette: List[AttributeInfo] = ()
    groupsAndProxies: List[GroupInfo] = ()
    paletteIsComplete: bool = False
    groupsToDelete: List[POM_object] = ()
    deletePalette: bool = False


@dataclass
class ModelViewProxyInfo(TcBaseObj):
    """
    An input for creating, updating or deleting model view proxy objects.  When creating a new model view
    proxy(Fnd0ModelViewProxy), the caller must specify the business object type ('boType') of the new model view proxy.
     During update, 'boType' is left empty and a reference to the model view proxy ('modelView') to be updated is
    specified.
    
    :var clientId: The use of this value is optional, but recommended.  It is used for mapping server errors and for
    correlating data in the response to the inputs.  The caller should populate a 'clientId' for each
    ModelViewProxyInfo with a value that is unique within the input set.   The value is not interpreted or manipulated
    internally by the server.
    :var modelView: The Fnd0ModelViewProxy to be updated.  A value is required for update or delete action cases,
    otherwise (create case) this value should be empty.
    :var boType: The business object type (its internal string name) of a model view proxy to be created.  This value
    is optional for creating a new model view proxy, and must be a Fnd0ModelViewProxy or subtype.  If no value is
    provided but also no modelView is provided, then the boType default value will be set to "Fnd0ModelViewProxy".  For
    update of an existing model view proxy, this value may be empty.
    :var deleteProxy: If true, specifies that this existing proxy should be deleted if possible. For update of an
    existing model view proxy or create of a new proxy, this value should be left at a default of false.
    :var owningModel: The owning object which manages the actual CAD model view for which the proxy is being created.
    This owning object will also control the lifecycle of the new proxy object. Required for creating a new model view
    proxy.  If an existing model view proxy is being updated, then this value may be empty.
    :var attrList: A list of name-value pairs (string, list of strings) used to specify additional property data for
    model view proxies.  All values are specified as strings, the caller is responsible for generating the correct
    string representation of each value passed.  For business object values, the UID of the object is used.
    :var thumbnailFile: The File to associate with the model view proxy to give a thumbnail of the model view. Value
    can be null.
    :var hiresImageFile: The File to associate with the model view proxy to give a higher resolution (versus the
    thumbnail) image of the model view. Value can be null.
    :var guardObjLastModifiedDate: If true, causes the update or delete of a model view proxy to check the proxy's last
    modified date to avoid data overwrite.
    :var objLastModifiedDateGuard: Modification date guard is used if 'guardObjLastModifiedDate' is set to true. Object
    update or delete will abort if the last modified date of the object is greater than 'objLastModifiedDateGuard'.
    """
    clientId: str = ''
    modelView: Fnd0ModelViewProxy = None
    boType: str = ''
    deleteProxy: bool = False
    owningModel: WorkspaceObject = None
    attrList: List[AttributeInfo] = ()
    thumbnailFile: ImanFile = None
    hiresImageFile: ImanFile = None
    guardObjLastModifiedDate: bool = False
    objLastModifiedDateGuard: datetime = None


@dataclass
class OwningModelAndCadLmd(TcBaseObj):
    """
    OwningModelAndCadLmd is used to identify when the CAD model views have last been modified. For each owning model
    object being passed in a OwningModelAndCadLmd input, all the owned model view proxies should be have their CAD last
    modified date synchronized to this date which is being provided.
    
    :var owningModel: The owning object which manages the actual CAD  model views which have a new date time. Required.
    :var cadLastModifiedDate: The date time at which at least some CAD model views have been most recently updated, and
    hence the date time to which the associated proxy objects should have their CAD last modified date set. Required.
    :var owningDataset: The owning dataset which manages the actual CAD  model views which have a new date time.
    Optional.
    """
    owningModel: WorkspaceObject = None
    cadLastModifiedDate: datetime = None
    owningDataset: Dataset = None


@dataclass
class CreateOrUpdateMVPaletteResponse(TcBaseObj):
    """
    The response contains a map of input caller specified 'clientId' values and the corresponding objects that were
    created or updated. The service data contains a list of added, updated, or deleted objects and it also contains a
    list of any errors which occurred within the call.
    
    :var clientIDMap: Map(string, Fnd0ModelViewPalette or Fnd0ModelViewGroup) of the various input 'clientId' to
    corresponding objects (Fnd0ModelViewPalette or Fnd0ModelViewGroup) that were created. Any input Info structures for
    which a 'clientId' wasn't provided will not have objects populated here but only within the 'serviceData'.
    :var serviceData: Contains a list of added, updated, or deleted objects.  Also contains list of any errors which
    occurred within the call.
    """
    clientIDMap: StringToBoMap = None
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateModelViewProxiesResponse(TcBaseObj):
    """
     The response contains a map of input caller specified client ID values and the corresponding objects that were
    created or updated. The service data contains a list of added, updated, or deleted objects and it also contains a
    list of any errors which occurred within the call.
    
    :var clientIDMap: Map (string, Fnd0ModelViewProxy) of the various input client IDs to corresponding objects
    (Fnd0ModelViewProxy) that were created.
    :var serviceData: Contains a list of added, updated, or deleted objects.  Also contains list of any errors which
    occurred within the call.
    """
    clientIDMap: StringToBoMap = None
    serviceData: ServiceData = None


"""
Generic map of a string to a business object.
"""
StringToBoMap = Dict[str, BusinessObject]



from __future__ import annotations

from tcsoa.gen.Workflow._2019_06.Workflow import CreateOrUpdatePALResponse, CreateUpdatePALInput, CreateOrUpdateTemplateResponse, CreateUpdateHandlerInput, GetRegisteredHandlerResponse, CreateOrUpdateHandlerResponse, CreateUpdateTemplateInput
from typing import List
from tcsoa.utils import TcService


class WorkflowService(TcService):

    @classmethod
    def getRegisteredHandlers(cls) -> GetRegisteredHandlerResponse:
        """
        This operation returns the list of names of registered workflow handlers. The names returned by this operation 
        are used as an input for creating the handler instance(s).
        
        Use cases:
        In Active Workspace Client, this operation will be used in a Workflow Designer sub-location for following cases:
        
        1.    Creating a new workflow handler instance (EPMHandler) for specified workflow template (EPMTaskTemplate).
        Here user will use the name of the handler (shown as the dropdown list) returned from this service operation.
        """
        return cls.execute_soa_method(
            method_name='getRegisteredHandlers',
            library='Workflow',
            service_date='2019_06',
            service_name='Workflow',
            params={},
            response_cls=GetRegisteredHandlerResponse,
        )

    @classmethod
    def createOrUpdateHandler(cls, input: List[CreateUpdateHandlerInput]) -> CreateOrUpdateHandlerResponse:
        """
        Creates a new instance or updates an existing instance of workflow handler(s). User can add a new workflow
        handler or update existing handler added on a workflow template in a Workflow Designer application.
        
        Use cases:
        In Active Workspace Client, this operation can be used in a Workflow Designer sub-location for following cases:
        
        1.    Creating a new workflow handler instance (EPMHandler) for specified workflow template (EPMTaskTemplate).
        2.    Modifying the exisitng workflow handler instance (EPMHandler) for specified workflow template
        (EPMTaskTemplate).
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateHandler',
            library='Workflow',
            service_date='2019_06',
            service_name='Workflow',
            params={'input': input},
            response_cls=CreateOrUpdateHandlerResponse,
        )

    @classmethod
    def createOrUpdatePAL(cls, input: List[CreateUpdatePALInput]) -> CreateOrUpdatePALResponse:
        """
        Creates or updates or deep copies EPMAssignmentList objects - PAL (Process Assignment List). Newly created
        EPMAssignmentList objects reference a workflow template. These EPMAssignmentList objects can then be used by
        workflow handlers to assign tasks during workflow execution. In Active Workspace, EPMAssignmentList objects can
        be used for assignments in &lsquo;Assignments&rsquo; tab. The &lsquo;Submit to workflow&rsquo; panel can be
        used to perform assignment using EPMAssignmentList configured in the workflow.
        
        Use cases:
        In Active Workspace Client, this operation can be used in &lsquo;Assignment List&rsquo; panel for following
        cases:
        
        1.    Creating new process assignment list (EPMAssignmentList) for specified workflow template
        (EPMTaskTemplate).
        2.    Modifiying resource assignments for a process assignment list object (EPMAssignmentList) . 
        3.    Create new process assignment list(EPMAssignmentList) based on an existing assignment list.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdatePAL',
            library='Workflow',
            service_date='2019_06',
            service_name='Workflow',
            params={'input': input},
            response_cls=CreateOrUpdatePALResponse,
        )

    @classmethod
    def createOrUpdateTemplate(cls, input: List[CreateUpdateTemplateInput]) -> CreateOrUpdateTemplateResponse:
        """
        Creates a new instance or updates an existing instance of workflow task template ( EPMTaskTemplate ). User can
        add a new workflow task template or update an existing task template in a workflow template in Workflow
        Designer application.
        
        Use cases:
        In Active Workspace Client, this operation can be used in a Workflow Designer sub-location for following cases:
        
        1.    Creating a new workflow task template instance (EPMTaskTemplate).
        2.    Modifying the existng workflow task template instance (EPMTaskTemplate) for specified workflow template
        (EPMTaskTemplate).
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateTemplate',
            library='Workflow',
            service_date='2019_06',
            service_name='Workflow',
            params={'input': input},
            response_cls=CreateOrUpdateTemplateResponse,
        )



from __future__ import annotations

from tcsoa.gen.Cad._2016_03.StructureManagement import BoolMap, FindModelViewsInStructureResponse
from tcsoa.gen.BusinessObjects import BusinessObject, ConfigurationContext
from tcsoa.gen.Cad._2016_03.DataManagement import CreateOrUpdateMVPaletteResponse, ModelViewPaletteInfo, OwningModelAndCadLmd, CreateOrUpdateModelViewProxiesResponse, ModelViewProxyInfo
from typing import List
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def createOrUpdateModelViewPalette(cls, mvPaletteInfo: List[ModelViewPaletteInfo]) -> CreateOrUpdateMVPaletteResponse:
        """
        Creates, updates or deletes a model view palette (Fnd0ModelViewPalette) object and its groups. The model view
        palette supports Visualization tools in creating and managing model view groups (Fnd0ModelViewGroup) during
        creation of a Disclosure object (an Item or Workset that discloses installation assembly designs). The
        disclosed model view proxy objects may be grouped by this operation.
        Using this API, applications can create and update a list of objects in bulk. Providing better context and
        fewer calls from the CAD clients than otherwise would be achieved using standard object create and update
        service operations.
        
        Use cases:
        This API supports the following use cases
        
        Use Case 1: Creation of Model View Palette and Model View Groups for a Design Disclosure
        
        The operation can be used for supporting creation of a disclosure object. The disclosure object will most
        commonly be a Workset (Cpd0Workset subtype) but may also be a specific type of ItemRevision. The actual
        disclosure object may be pre-existing but would not be acting as a disclosure until this operation creates and
        attaches the desired list of disclosed Model View Proxy references.
        
        The purpose of a design disclosure is to act as a 3D equivalent of a 2D drawing of individual installation
        assemblies (IA.) This disclosure object will collect all geometry needed to show both the IA and its assembly
        PMI. After the various PMI and Model Views within the disclosure have been created, they must be "disclosed". 
        
        To be considered as disclosed by a disclosure object, a Model View Proxy must be referenced by a Model View
        Group (Fnd0ModelViewGroup)that is referenced by a Model View Palette (Fnd0ModelViewPalette) that is associated
        to the disclosure object.
        
        The 'paletteIsComplete' input must be set to true for creation of a Model View Palette, as it makes no sense to
        be false.
        
        Use Case 2: Update of Model View Palette and Model View Groups for a Design Disclosure
        If 'groupsToDelete' is set then see Use case 3 for details, else if 'paletteIsComplete' is set to true, then
        the system will:
          1) Process 'groupsAndProxies', creating or updating various Groups.
          2) Check the existing Group list on the Palette to determine if any existing Groups in the current Palette
        list are not in the input 'groupsAndProxies' input. If so, then set aside such existing Groups for deletion.
          3) Set the list of existing and newly created Groups (from step 1 above) onto the Palette.
          4) Now delete the now unreferenced Groups (from step 2) if any.
        
        If paletteIsComplete is set to false, the system will:
          1) process groupsAndProxies, creating or updating various Groups.
          2) Again, if groupsToDelete is set then see Use case 3 for details. This must be done before re-ordering the
        Palette's sequence of Groups in order for the input sequence order values to be understood correctly by the
        system.
          3) Set the list of existing and newly created Groups (from step 1 above) onto the Palette list of Groups
        (using any specified newOrderSequenceNumber if one was given. If any newOrderSequenceNumber is within the range
        of existing Groups, and that existing Group is not in the groupsAndProxies being given a new sequence number,
        then the existing group being replaced in that sequence number will be set aside to be placed at the end of the
        sequence.
          4) Now place any set-aside (due to be pushed out of place or being given a sequence value of 0) Groups at the
        end of the Palette's list of Groups.
        
        
        Use Case 3: Delete of existing Model View Palette and/or specific Groups within the Palette
        
        To delete an entire Palette and all its Groups, the caller will simply send both the disclosure and the
        deletePalette flag.
        The system will :
        - Verify write permission on the disclosure object in order to proceed.
        - Remove the relation between the disclosure object and the Model View Palette.
        - Delete the Model View Palette
        - Delete all the Model View Groups that were previously referenced by the Model View Palette
        
        
        
        To delete only specified Groups out of all those currently existing on a Palette, the caller will send the
        Groups to delete in the groupsToDelete list.
        The system will:
        - Note: write permission on the Model View Palette is required.
        - Remove all the specified Model View Groups from the reference list on the Model View Palette, preserving the
        order among the remaining Model View Groups referenced by the Palette.
        - Delete the now unreferenced Model View Groups.
        
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateModelViewPalette',
            library='Cad',
            service_date='2016_03',
            service_name='DataManagement',
            params={'mvPaletteInfo': mvPaletteInfo},
            response_cls=CreateOrUpdateMVPaletteResponse,
        )

    @classmethod
    def createOrUpdateModelViewProxies(cls, mvProxyInfos: List[ModelViewProxyInfo], updatedOwningModels: List[OwningModelAndCadLmd]) -> CreateOrUpdateModelViewProxiesResponse:
        """
        Creates, updates or deletes a set of model view proxy (Fnd0ModelViewProxy) objects. Supports CAD tools in
        creating and managing model view proxy objects during Part save. The model view proxy objects are each a proxy
        for a master model view in the Part's CAD file.
        Using this API, applications can create and update proxy objects in bulk, with better context and less calls
        from CAD clients than may otherwise be achieved using standard object create and update SOAs.
        
        Use cases:
        This API supports the following use cases:
        Use Case 1: Creation of new model view proxy 
         The following operation can be used for creating model view proxies for specified owning objects (usually
        ItemRevisions.)
        - Model view proxies have a model view CAD Id (fnd0ModelViewIdCAD) which Id is unique within the set of model
        view proxies associated to the same owning object.
        - During the model view proxies initial creation (but not during a subsequent save-as or revise), a clone
        stable ID is generated which can help in identifying equivalent proxy objects for cases where the
        fnd0ModelViewIdCAD wasn't tracked but rather the proxy object itself.
        - An optional thumbnail file may be identified that a CAD tool has generated for the actual CAD model view.
        - During the operation, the server creates and saves the new model view proxies in context of an already
        existing owning object. The operation returns the new objects to the caller.
        
        
        
        Use Case 2: Update of existing model view proxy 
        The following operation can be used for updating existing model view proxies.
        - Model View Proxies are found by applications via the fnd0OwnedModelViews property of workspaceObject objects.
        - The existing model view proxies can be updated using operation createOrUpdateModelViewProxies. The
        application specifies which model view proxies are to be updated.  Note: the business object type ('boType')
        and owning object ('owningModel') do not need to be set on the input because they are already known to the
        model view proxy and cannot be changed.  The application sets changed property values.
        
        
        
        Use Case 3: Delete of existing model view proxy
        The operation can be used for deleting existing model view proxies due to either the CAD designer removing the
        actual model view or due to a decision that there is no need for that model view to have a proxy any longer.
        - Model View Proxies are found by applications via the fnd0OwnedModelViews property of certain types of
        WorkspaceObject objects (currently this property is only available at  ItemRevision ).
        - The existing model view proxies can be deleted using operation createOrUpdateModelViewProxies. The
        application specifies which model view proxies are to be modified ('modelView') and that they are to be deleted
        ('deleteProxy' set to true.)  The server will attempt to delete the model view proxy and if successful, the
        deleted object list will be updated on the ServiceData of the response.
        
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateModelViewProxies',
            library='Cad',
            service_date='2016_03',
            service_name='DataManagement',
            params={'mvProxyInfos': mvProxyInfos, 'updatedOwningModels': updatedOwningModels},
            response_cls=CreateOrUpdateModelViewProxiesResponse,
        )


class StructureManagementService(TcService):

    @classmethod
    def findModelViewsInStructure(cls, disclosure: BusinessObject, structureScope: List[BusinessObject], configurationContext: ConfigurationContext, withDisclosureIntent: List[str], options: BoolMap) -> FindModelViewsInStructureResponse:
        """
        Finds Model View Proxy ( Fnd0ModelViewProxy) objects associated to any objects within the specified structure.
        This operation is most often used when creating a disclosure object.
        
        Objects that act as a disclosure are Items or Worksets that have a relation of Fnd0DisclosingObject to the
        actual design objects being disclosed and hence intend to have a list of disclosed Model View Proxy objects.
        
        Use cases:
        This API supports the following use cases:
        Use Case 1: Creation of a new design disclosure
        
        The operation can be used for supporting creation of a disclosure object. The disclosure object will most
        commonly be a Workset (Cpd0Workset subtype) but may also be a specific type of Item Revision. The actual
        disclosure object may be pre-existing but would not be acting as a disclosure until this operation creates and
        attaches the desired list of disclosed model view references.
        
        The purpose of a design disclosure is to act as a 3D equivalent of a 2D drawing. This disclosure object will
        collect all geometry including background geometry if necessary and PMI needed to show all views describing the
        detailed design for the object being disclosed. The disclosure content may be organized into the following item
        revisions or subsets (depending on the disclosure type):
        - Foreground content - actual installation assembly reference
        - Background content - context of a product into which the installation assembly is used.
        - Separately collected geometry - such as welds between the foreground and background objects. 
        
        
        
        To create the correct model view list (see the createOrUpdateModelViewPalette service operation), the client
        and user must first find candidate model view proxies from which to choose the necessary proxies to disclose.
        
        During the operation, a designer would create a Workset to collect all geometry needed to support  installation
        assembly PMI. It is done to collect assemblies being installed and where they are being installed so that PMI
        and model views associated with this combination can be authored and then disclosed. The following types of
        geometry may be collected in a single Workset:
        - Foreground Subset
        - Background Subset
        - ItemRevision (Weld Collector)
        
        
        
        - Multiple CAD Designers will be concurrently authoring PMI and Model Views (MVs) for disclosure at multiple
        levels of the sub-assemblies under the installation assembly. All the MVs authored at this time are
        undisclosed. However, some of them will be marked as a candidates for disclosure.
        - Owning model (ItemRevision) must exist prior to creating the MV proxy in teamcenter by a new service
        operation createOrUpdateModelViewProxies. The owing model will be specified as a request parameter for each MV
        proxy.
        
        
        
        - A visualization user begins Disclosure authoring by retrieving the above mentioned collector workset and
        making sure it has disclosing object (Fnd0DisclosingObject) relations to the actual installation item revisions
        whose design is being disclosed. They will then use the findModelViewsInStructure operation to find model views
        that are marked as candidates for disclosure and then use a third operation (createOrUpdateModelViewPalette) to
        create the list of actually disclosed model views to be persisted for the Disclosure being updated. 
        
        
        Note:
        Candidate Disclosed Model Views will be retrieved from objects in the Foreground Installation Assembly realized
        subset content, and the Weld Collector Item Revision. They will not belong to either Standard Parts or
        Background Geometry unless the 'withDisclosureIntent'  value is appropriately set or the "skipBackgroundScope"
        option is set to false.
        
        
        Use Case 2: Update of an existing design disclosure
        
        The operation can also be used for supporting the update of a disclosure object. In this use case, the
        disclosed list of model view proxy references is compared with the current design content. Some proxies
        previously disclosed may not be found in the new structure, or some of the proxies may have revised owning
        objects and the service provides this type of information to the caller.
        
        To create the correct model view list (see the createOrUpdateModelViewPalette service operation), the client
        and user must first find candidate model view proxies to remove from the existing list and which candidate
        model view proxies to add to the existing disclosure list.
        
        During the operation :
        
        A visualization user begins Disclosure update by retrieving the above mentioned collector workset. They will
        then use this findModelViewsInStructure operation to find new model views that are marked as candidates for
        disclosure and then use a third operation (createOrUpdateModelViewPalette) to update the list of actually
        disclosed model views to be persisted for the Disclosure being updated. While calling
        findModelViewsInStructure, if the disclosure is passed in as the 'structureScope', then the "compareWithMVList"
        option may also be set. This will request the server to also compare the found model view proxy objects in the
        structure against those model view proxies currently listed as disclosed by the disclosure object, calling out
        ( in 'unfoundFromModelViewList') those which are in the model view list but no longer found in the structure.
        Also returned when known are proposed replacement proxy objects for each such now missing proxy ( in
        'possibleMatching' .)
        """
        return cls.execute_soa_method(
            method_name='findModelViewsInStructure',
            library='Cad',
            service_date='2016_03',
            service_name='StructureManagement',
            params={'disclosure': disclosure, 'structureScope': structureScope, 'configurationContext': configurationContext, 'withDisclosureIntent': withDisclosureIntent, 'options': options},
            response_cls=FindModelViewsInStructureResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, POM_object
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ValidationService(TcService):

    @classmethod
    def getValidationResults2(cls, validationAgentRevObjects: List[POM_object], valDataObjects: List[POM_object], selectedObjects: List[BusinessObject], targetObjects: List[POM_object]) -> ServiceData:
        """
        This operation will return validation result objects in the ServiceData structure. This operation will take the
        given validation agents, validation data objects, selected objects, and/or validation target objects to build
        the search criteria to find the validation result objects. Validation agents represent applications that were
        used to generate the validation result objects. Validation data objects represent the specialized validation
        checkers that are referenced by the validation result objects.  The selected objects represent the owning
        business objects of the validation result objects. The target objects represent the validation target objects
        that are referenced by the validation result objects.
        
        Use cases:
        A validaiton result object is generated for a business object by a validation agent using the specialized
        validation logic that is defined by a validation data object. For example, the NX Check Mate validation agent
        will execute NX Check Mate tests to generate validation result objects for an NX part item revision object
        using the underlying UGMaster dataset object as a validation target.  This operation will be used to search for
        the validation result objects that a user will be interested in by reducing the search scope using the given
        input objects.
        """
        return cls.execute_soa_method(
            method_name='getValidationResults2',
            library='Internal-Validation',
            service_date='2010_04',
            service_name='Validation',
            params={'validationAgentRevObjects': validationAgentRevObjects, 'valDataObjects': valDataObjects, 'selectedObjects': selectedObjects, 'targetObjects': targetObjects},
            response_cls=ServiceData,
        )

    @classmethod
    def getValidationTargets(cls, validationAgentRevObjects: List[POM_object], valDataObjects: List[POM_object], selectedObjects: List[BusinessObject]) -> ServiceData:
        """
        This operation will execute validation business logic of the given validation agent to generate
        ValidationResult objects. This operation will take the given validation agent, validation data objects, the
        selected business objects, and validation target objects to generate the ValidationResult objects. The given
        validation agent represents the validation application that will be responsible for generating the validation
        result objects. Validation data objects represent the specialized validation checkers that are referenced by
        the validation result objects.  The selected objects represent the owning business objects of the validation
        result objects and will be used to collect the associated validation target objects to be combined with the
        given 'targetObjects' list for validation evaluation. The target objects represent the validation target
        objects that are referenced by the generated validation result objects.
        
        Use cases:
        This operation can be used to find out the list of associated validation target objects that will be evaluated
        by validation agents and checkers in validation operations.
        """
        return cls.execute_soa_method(
            method_name='getValidationTargets',
            library='Internal-Validation',
            service_date='2010_04',
            service_name='Validation',
            params={'validationAgentRevObjects': validationAgentRevObjects, 'valDataObjects': valDataObjects, 'selectedObjects': selectedObjects},
            response_cls=ServiceData,
        )

    @classmethod
    def runValidation(cls, validationAgentRev: POM_object, valDataObjects: List[POM_object], selectedObjects: List[BusinessObject], targetObjects: List[POM_object]) -> ServiceData:
        """
        This operation will execute validation business logic of the given validation agent to generate
        ValidationResult objects.  This operation will take the given validation agent, validation data objects, the
        selected business objects, and validation target objects to generate the ValidationResult objects. The given
        validation agent represents the validation application that will be responsible for generating the validation
        result objects. Validation data objects represent the specialized validation checkers that are referenced by
        the validation result objects.  The selected objects represent the owning business objects of the validation
        result objects and will be used to collect the associated validation target objects to be combined with the
        given 'targetObjects' list for validation evaluation. The target objects represent the validation target
        objects that are referenced by the generated validation result objects.
        
        Use cases:
        A Validation Administrator or System Administrator uses the Validation Manager application to create validation
        agents and checkers. The applications used to validate the data must be configured to perform validations for
        the defined objects. You do this by creating validation agents and checkers for each desired application. To
        enable object validation functionality, validation agents and checkers must be defined within the relevant
        applications.
        
        This operation will enable a user to use a given validation agent to generate the ValidationResult objects by
        invoking the validation business logic specified by a list of validation checkers on the selected business
        objects or on the given validation target objects. The ValidationResult objects will show the Pass or Fail
        result status of a business object for each individual validation checker and each validation target object.
        """
        return cls.execute_soa_method(
            method_name='runValidation',
            library='Internal-Validation',
            service_date='2010_04',
            service_name='Validation',
            params={'validationAgentRev': validationAgentRev, 'valDataObjects': valDataObjects, 'selectedObjects': selectedObjects, 'targetObjects': targetObjects},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.ImportExport._2012_09.FileImportExport import ImportFromApplicationInputData3
from tcsoa.gen.ImportExport._2011_06.FileImportExport import ImportFromApplicationResponse1
from typing import List
from tcsoa.utils import TcService


class FileImportExportService(TcService):

    @classmethod
    def importFromApplication(cls, inputs: List[ImportFromApplicationInputData3]) -> ImportFromApplicationResponse1:
        """
        This operation imports a Requirement Specification document containing Requirement and Paragraph objects. It
        creates Requirement / Paragraph objects and associated data (FullText for each created Item to store the
        content from the document and IMAN_Specification relation between FullText and each created Item). The input
        structure for this operation contains file meta data information, type of specification elements (SpecElement)
        to be created, application format of the MS Word document being imported, keyword parsing options to be used
        during import, live or static import mode to be used for import, option to import as new specification or under
        the selected BOMLine object and description to be set on the Item once imported.
        """
        return cls.execute_soa_method(
            method_name='importFromApplication',
            library='ImportExport',
            service_date='2012_09',
            service_name='FileImportExport',
            params={'inputs': inputs},
            response_cls=ImportFromApplicationResponse1,
        )
